
/*********************************************************************************
 *                                                                               *
 * LaVache.cpp                                                                   *
 *                                                                               *
 * Copyright (C) 2000/2002 Fasouil                                               *
 *                                                                               *
 *                                                                               *
 * This program is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU General Public License                   *
 * as published by the Free Software Foundation; either version 2                *
 * of the License, or (at your option) any later version.                        *
 *                                                                               *
 *                                                                               *
 * This program is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
 * GNU General Public License for more details.                                  *
 *                                                                               *
 *                                                                               *
 * You should have received a copy of the GNU General Public License             *
 * along with this program; if not, write to the Free Software                   *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    *
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html                *
 *                                                                               *
 *                                                                               *
 * The author can be reached at fasouil@hotmail.com                              *
 *********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "Structures.h"
#include "LaVache.h"



///////////////////////////////////////////////
#define SafeDelete(X) {if (X) delete X; X=NULL;}

CLaVache::CLaVache()
{
        m_bInit=false;

        m_pbPossibleNormal=NULL;
        m_psCountKeyLineNormal=NULL;
        m_pnPositionLineNormal=NULL;
        m_ppsAdressKeyLineNormal=NULL;
        m_pcKeyLineNormal=NULL;
        m_pnTamponChargeCapture=NULL;
        m_psCountKeyLinePrevious=NULL;
        m_pbPossiblePrevious=NULL;
        m_pnPositionLinePrevious=NULL;
        m_ppsAdressKeyLinePrevious=NULL;
        m_pcKeyLinePrevious=NULL;
        m_psKeyMaskPurple=NULL;
        m_pnArrayMaskPurple=NULL;
        m_psInterliveSort=NULL;
        m_psInterPixelNormal=NULL;
        m_psInterPixelPrevious=NULL;
        m_pnValueDiffPixelLine=NULL;


        m_pnArrayMaskPurple=new unsigned __int64[128*256];
        if (!m_pnArrayMaskPurple)
                return;

        m_psKeyMaskPurple=new short[128*256];
        if (!m_psKeyMaskPurple)
                return;

        m_pnArrayMaskChromi=new unsigned __int64[128*256];
        if (!m_pnArrayMaskChromi)
                return;

        m_psKeyMaskChromi=new short[128*256];
        if (!m_psKeyMaskChromi)
                return;

        m_cKeyLine=new unsigned char[256];
        if(!m_cKeyLine)
      return;

        m_pnMemoDistance=new int[CURRENTY*CURRENTY];
        if(!m_pnMemoDistance)
      return;

        m_pcKeyAlreadyFound=new char[128*256+1];
        if(!m_pcKeyAlreadyFound)
      return;

        m_pbPossibleNormal=new bool[CURRENTY*CURRENTY];
        if(!m_pbPossibleNormal)
                return;

        m_pbPossiblePrevious=new bool[CURRENTY*CURRENTY];
        if(!m_pbPossiblePrevious)
      return;
}


CLaVache::~CLaVache()
{
        SafeDelete(m_pcKeyLinePrevious);
        SafeDelete(m_ppsAdressKeyLinePrevious);
        SafeDelete(m_pnPositionLinePrevious);
        SafeDelete(m_psCountKeyLinePrevious);
        SafeDelete(m_pbPossiblePrevious);
        SafeDelete(m_pnTamponChargeCapture);
        SafeDelete(m_pcKeyLineNormal);
        SafeDelete(m_pnPositionLineNormal);
        SafeDelete(m_ppsAdressKeyLineNormal);
        SafeDelete(m_psCountKeyLineNormal);
        SafeDelete(m_cKeyLine);
        SafeDelete(m_pnMemoDistance);
        SafeDelete(m_pcKeyAlreadyFound);
        SafeDelete(m_psInterliveSort);
        SafeDelete(m_psInterPixelNormal);
        SafeDelete(m_psInterPixelPrevious);
        SafeDelete(m_pnValueDiffPixelLine);
        SafeDelete(m_pnArrayMaskChromi);
        SafeDelete(m_psKeyMaskChromi);
        SafeDelete(m_pnArrayMaskPurple);
        SafeDelete(m_psKeyMaskPurple);
}

//debug
double CLaVache::GetInfo()
{
        return m_fHeightBlackDynam;
        return m_dInfo;
}


bool CLaVache::GetPhase()
{
        return (m_bPhase);
}


bool CLaVache::IsInit()
{
        return (m_bInit);
}

void CLaVache::Destroy()
{
        m_bInit=false;
        SafeDelete (m_psCountKeyLineNormal);
        SafeDelete (m_pnPositionLineNormal);
        SafeDelete (m_ppsAdressKeyLineNormal);
        SafeDelete (m_pcKeyLineNormal);
        SafeDelete (m_psCountKeyLinePrevious);
        SafeDelete (m_pnPositionLinePrevious);
        SafeDelete (m_ppsAdressKeyLinePrevious);
        SafeDelete (m_pcKeyLinePrevious);
        SafeDelete (m_psInterliveSort);
        SafeDelete (m_psInterPixelNormal);
        SafeDelete (m_psInterPixelPrevious);
        SafeDelete (m_pnValueDiffPixelLine);
}

bool CLaVache::Init()
{
        m_bInit=false;
        m_dInfo=0;
        m_nSizeBufferX=0;
        m_nSizeBufferY=0;
        m_bPhase=false;
        m_nMarkChromi=0;
        m_nKeyFound=0;
        m_bKeyFound=false;
        m_nLengthTestChromi=0;
        m_nPositionTestChromi=0;
        m_bInit=false;
        m_bUsedLoadImage=false;
        m_bIsOdd=false;
        m_nTypeTryPrevious=0;
        m_bKeyRepeatLock=false;
        m_bKeyRepeatRead=false;
        m_nPositionKeyRepeat=0;
        m_nTestDone=0;
        m_nPositionKeyRepeatFound=0;
        m_nBadKeySynchrone=0;
        m_nGoodKeySynchrone=0;
        m_nBadKeySynchroneOld=0;
        m_fScrambSignalDetect=0;
        m_nRateGoodProba=0;
        m_nTestNormalUser=0;
        m_nTestPreviousUser=0;
        m_nLengthTestPurple=0;
        m_nPositionTestPurple=0;
        m_nMarkPurple=0;
        m_bFilterPurple=false;
        m_bFilterSECAM=0;
        m_bFilterAverage = false;
        m_nHeightBlackBorder=0;
        m_fHeightBlackDynam=0;


        m_nProbaNormalDynam=200;        //200
        m_nProbaNormalLimitSup=m_nProbaNormalDynam*COEFFILTERLIMIT;
        m_nProbaNormalLimitInf=0;

        m_nProbaPreviousDynam=400;      //400
        m_nProbaPreviousLimitSup=m_nProbaPreviousDynam*COEFFILTERLIMIT;
        m_nProbaPreviousLimitInf=0;


        ZeroMemory(m_pbPossiblePrevious,sizeof(*m_pbPossiblePrevious)*CURRENTY*CURRENTY);
        ZeroMemory(m_pnLineCurent288,sizeof(m_pnLineCurent288));
        ZeroMemory(m_pnLineNext32,sizeof(m_pnLineNext32));
        ZeroMemory(m_psSortLineCurrent,sizeof(m_psSortLineCurrent));
        ZeroMemory(m_pnLineOldSort288,sizeof(m_pnLineOldSort288));
        ZeroMemory(m_bOldKeyFound,sizeof(m_bOldKeyFound));
        ZeroMemory(m_pnColumnOldSortValue,sizeof(m_pnColumnOldSortValue));
        ZeroMemory(m_pnColumnOldSelect,sizeof(m_pnColumnOldSelect));
        ZeroMemory(m_psKeyRepeat,sizeof(m_psKeyRepeat));
        ZeroMemory(m_pnLineSort288,sizeof(m_pnLineSort288));

        if (!LoadFileKey())
                return m_bInit;

        if (!LoadFileChromi())
                return m_bInit;

        if (!LoadFilePurple())
                return m_bInit;

        if (!LoadFilePossibleNormal())
                return m_bInit;

        if (!LoadFileLineNormal())
                return m_bInit;

        if (!LoadFilePossiblePrevious())
                return m_bInit;

        if(!LoadFileLinePrevious())
                return m_bInit;

        m_psInterliveSort=new short[max(m_nTestNormalMaxi,m_nTestPreviousMaxi)];
        if (!m_psInterliveSort)
                return m_bInit;

        m_psInterPixelNormal=new short[max(m_nTestNormalMaxi,m_nTestPreviousMaxi)];
        if (!m_psInterPixelNormal)
                return m_bInit;

        m_psInterPixelPrevious=new short[max(m_nTestNormalMaxi,m_nTestPreviousMaxi)];
        if (!m_psInterPixelPrevious)
                return m_bInit;

        m_pnValueDiffPixelLine=new int[max(m_nTestNormalMaxi,m_nTestPreviousMaxi)];
        if (!m_pnValueDiffPixelLine)
                return m_bInit;

        m_bInit=true;
        return m_bInit;
}


void CLaVache::SetSizeBuffer(int nSizeBufferX,int nSizeBufferY)
{
        m_nSizeBufferX=nSizeBufferX;
        m_nSizeBufferY=nSizeBufferY;
}

bool CLaVache::UpdatePhase()
{
        const int nHeighStart=60;
        const int nHeighEnd=120;
        unsigned __int64 maskU =0x0000FF000000FF00ULL;
        unsigned __int64 maskV =0xFF000000FF000000ULL;
        unsigned __int64 mask00=0x0000000000000000ULL;

        int py;
        int NbU=0,NbV=0;
        int * pLine1;
        int * pLine2;
        int nPhase0=0;
        int nPhase1=0;

          __asm__ __volatile__ (
                "movq   %0,%%mm2         \n\t"
                "movq   %1, %%mm6        \n\t"
                "movq   %2, %%mm7        \n\t"
                : :
                 "m" (mask00),"m" (maskU), "m"(maskV)
                : "memory" );


        //calcul phase of trame
        for (py=nHeighStart;py<nHeighEnd-2;py+=2)
        {
                pLine1=m_pnLineCurent288[py];           //line
                pLine2=m_pnLineCurent288[py+1]; //line+1

                  __asm__  __volatile__ (

                        "movl            %0, %%edi \n\t"
                        "movl            %1, %%esi \n\t"

                        "movl            $2, %%ecx   \n\t"      //2 loop
                        "pxor    %%mm4, %%mm4        \n\t"
                        "pxor    %%mm5, %%mm5        \n\t"

//                        "jmp             UP.AutrePixel3 \n\t"
                        "jmp 0f \n\t"
                        ".align 16                   \n\t"
//                "UP.AutrePixel3:                \n\t"
                        "0: \n\t"

                        "movq    (%%edi), %%mm0      \n\t"      //mm0=*pLine1
                        "movq    (%%esi), %%mm1      \n\t"      //mm1=*pLine2

                        "psubb   %%mm1, %%mm0        \n\t"              //mm0=mm0-mm1 (b)
                        "movq    %%mm0, %%mm1        \n\t"              //mm1=mm0

                        "pand    %%mm6, %%mm0        \n\t"              //mm0=mm0&maskU
                        "pand    %%mm7, %%mm1        \n\t"              //mm1=mm1&maskV

                        "paddb   %%mm0, %%mm4        \n\t"              //mm4+=mm0 (b)
                        "paddb   %%mm1, %%mm5        \n\t"              //mm5+=mm1 (b)

                        "movq    8(%%edi), %%mm0     \n\t"      //mm0=*pLine1
                        "movq    8(%%esi), %%mm1     \n\t"      //mm1=*pLine2

                        "psubb   %%mm1, %%mm0        \n\t"              //mm0=mm0-mm1 (b)
                        "movq    %%mm0, %%mm1        \n\t"              //mm1=mm0

                        "pand    %%mm6, %%mm0        \n\t"              //mm0=mm0&maskU
                        "pand    %%mm7, %%mm1        \n\t"              //mm1=mm1&maskV

                        "addl            $16, %%esi  \n\t"
                        "paddb   %%mm0, %%mm4        \n\t"              //mm4+=mm0 (b)

                        "addl            $16, %%edi  \n\t"
                        "paddb   %%mm1, %%mm5        \n\t"              //mm5+=mm1 (b)

                        "decl            %%ecx       \n\t"
//                        "jnz             UP.AutrePixel3\n\t"                      //----
                        "jnz 0b \n\t"

                        "movq    %%mm4, %%mm0        \n\t"              //mm0=mm4
                        "movq    %%mm5, %%mm1        \n\t"              //mm1=mm5

                        "psrlq   $32, %%mm4          \n\t"              //mm4=mm4>>32
                        "psrlq   $32, %%mm5          \n\t"              //mm5=mm5>>32

                        "paddd   %%mm4, %%mm0        \n\t"              //mm0+=mm4 (dw)
                        "paddd   %%mm5, %%mm1        \n\t"              //mm1+=mm5 (dw)

                         "movd   %%mm0,%2              \n\t"
                         "movd   %%mm1,%3              \n\t"

                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                     "m" (pLine1), "m" (pLine2), "m" (NbU), "m" (NbU)

                   : "%ecx", "%edi", "%esi", "memory" // CHECKASM: clobber list
                );

                if (!NbU)
                        nPhase0++;
                if (!NbV)
                        nPhase1++;

        }

        if (nPhase0<nPhase1)
                m_bPhase=false;
        else
                m_bPhase=true;                 

        __asm__ __volatile__ ("emms");                              // output MMX mode

        return (true);
}



//UpdateMarkChromi for BT848
//use chrominance bug
bool CLaVache::UpdateMarkChromi()
{
        if (m_bKeyFound)
                return false;   //already found

        unsigned __int64 maskU =0x0000FF000000FF00ULL;
        unsigned __int64 maskV =0xFF000000FF000000ULL;
        unsigned __int64 mask00=0x0000000000000000ULL;

        int py;
        int NbU=0,NbV=0;
        int * pLine1;
        int * pLine2;
        int * pLine3;
        int * pLine4;
        int nIncrement1=0;
        int nIncrement2=0;
        unsigned __int64 nMarkTest1=0;
        unsigned __int64 nMarkTest2=0;

        if (!m_bPhase)
                nIncrement1=1;
        else
                nIncrement2=1;

          __asm__ __volatile__ (
                "movq    %0,  %%mm2        \n\t"
                "movq    %1,  %%mm6        \n\t"
                "movq    %2,  %%mm7        \n\t"
                : : "m" (mask00),"m" (maskU), "m"(maskV) : "memory"
        );

        //Calcul mark of chrominance 
        for (py=m_nPositionTestChromi;py<m_nPositionTestChromi+m_nLengthTestChromi;py+=2)
        {

                pLine1=m_pnLineCurent288[py+nIncrement1];
                pLine2=m_pnLineCurent288[py+nIncrement2];
                pLine3=m_pnLineCurent288[py+1+nIncrement1];
                pLine4=m_pnLineCurent288[py+1+nIncrement2];

                  __asm__  __volatile__ (
                        "movl            %0, %%edi \n\t"
                        "pxor    %%mm4, %%mm4        \n\t"

                        "movl            %2, %%esi \n\t"
                        "movl            $8, %%ecx   \n\t"

//                        "jmp             UMC.AutrePixel1 \n\t"
                        "jmp 0f \n\t"
                        ".align 16                   \n\t"
//                "UMC.AutrePixel1:                \n\t"
                        "0: \n\t"

                        //U

                        "movq    (%%esi), %%mm1      \n\t"      //mm1=*pLine3
                        "movq    (%%edi), %%mm0      \n\t"      //mm0=*pLine1

                        "psubb   %%mm1, %%mm0        \n\t"              //mm0=mm0-mm1 (b)
                        "pand    %%mm6, %%mm0        \n\t"              //mm0=mm0&maskU


                        "pcmpeqb %%mm2, %%mm0        \n\t"              //mm0>?mask00 (b)
                        "paddb   %%mm0, %%mm4        \n\t"              //mm4+=mm0 (b)

                        "movq    8(%%esi), %%mm1     \n\t"      //mm1=*pLine3
                        "movq    8(%%edi), %%mm0     \n\t"      //mm0=*pLine1

                        "psubb   %%mm1, %%mm0        \n\t"              //mm0=mm0-mm1 (b)
                        "pand    %%mm6, %%mm0        \n\t"              //mm0=mm0&maskU

                        "addl            $16, %%esi  \n\t"
                        "pcmpeqb %%mm2, %%mm0        \n\t"              //mm0>?mask00 (b)

                        "addl            $16, %%edi  \n\t"
                        "paddb   %%mm0, %%mm4        \n\t"              //mm4+=mm0 (b)

                        "decl            %%ecx       \n\t"
//                        "jnz         UMC.AutrePixel1 \n\t"                      //----
                        "jnz 0b \n\t"

                        "movl            %1, %%edi \n\t"
                        "pxor    %%mm5, %%mm5        \n\t"

                        "movl            %3, %%esi \n\t"
                        "movl            $8, %%ecx   \n\t"

//                        "jmp             UMC.AutrePixel2 \n\t"
                        "jmp 1f \n\t"
                        ".align 16                   \n\t"
//                "UMC.AutrePixel2:                \n\t"
                        "1: \n\t"

                        //V

                        "movq    (%%esi), %%mm1      \n\t"      //mm1=*pLine4
                        "movq    (%%edi), %%mm0      \n\t"      //mm0=*pLine2

                        "psubb   %%mm1, %%mm0        \n\t"              //mm0=mm0-mm1 (b)
                        "pand    %%mm7, %%mm0        \n\t"              //mm0=mm0&maskV

                        "pcmpeqb %%mm2, %%mm0        \n\t"              //mm0>?mask00 (b)
                        "paddb   %%mm0, %%mm5        \n\t"              //mm5+=mm0 (b)

                        "movq    8(%%esi), %%mm1     \n\t"      //mm1=*pLine4
                        "movq    8(%%edi), %%mm0     \n\t"      //mm0=*pLine2

                        "psubb   %%mm1, %%mm0        \n\t"              //mm0=mm0-mm1 (b)
                        "pand    %%mm7, %%mm0        \n\t"              //mm0=mm0&maskV

                        "addl            $16, %%esi  \n\t"
                        "pcmpeqb %%mm2, %%mm0        \n\t"              //mm0>?mask00 (b)

                        "addl            $16, %%edi  \n\t"
                        "paddb   %%mm0, %%mm5        \n\t"              //mm5+=mm0 (b)

                        "decl            %%ecx       \n\t"
//                        "jnz             UMC.AutrePixel2 \n\t"                      //----
                        "jnz 1b \n\t"

                        //addition
                        "movq    %%mm4, %%mm0        \n\t"              //mm0=mm4
                        "movq    %%mm5, %%mm1        \n\t"              //mm1=mm5

                        "psrlq   $32, %%mm4          \n\t"              //mm4=mm4>>32
                        "psrlq   $32, %%mm5          \n\t"              //mm5=mm5>>32

                        "paddb   %%mm4, %%mm0        \n\t"              //mm0+=mm4 (b)
                        "paddb   %%mm5, %%mm1        \n\t"              //mm1+=mm5 (b)

                        "movd   %%mm0,%4           \n\t"
                        "movd   %%mm1,%5           \n\t"


                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)
                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                    "m" (pLine1), "m" (pLine2),"m" (pLine3), "m" (pLine4), "m" (NbU), "m" (NbV)
                   : // CHECKASM: clobber list
                     "%edi", "%esi", "%ecx","memory"
                );

                NbU=(((NbU&0x0000FF00)>>8)+0x20)&0xff;
                NbV=(((NbV&0xFF000000)>>24)+0x20)&0xff;


                nMarkTest1=nMarkTest1<<2;
                if (NbU>10)
                        nMarkTest1|=1;

                nMarkTest2=nMarkTest2<<2;
                if (NbV>10)
                        nMarkTest2|=1;
        }
        
        __asm__ __volatile__ ("emms");                              // output off MMX mode

        //make the true mark
        nMarkTest1=(nMarkTest2<<nIncrement1|nMarkTest1<<nIncrement2)&0x7fffffffffffffffLL;

        if (nMarkTest1==0)
        {
                m_fScrambSignalDetect=(float)((m_fScrambSignalDetect*2.0+0.0)/3.0);
                return false;
        }

        m_fScrambSignalDetect=(float)((m_fScrambSignalDetect*2.0+1.0)/3.0);

        m_nMarkChromi=nMarkTest1;

        return true;
}


//UpdateMarkPurple for BT878
//use purple line
 bool CLaVache::UpdateMarkPurple()
{
        if (m_bKeyFound)
                return false;   //already found

        int nPosition;
        unsigned __int64 nMarkTest1=0ULL;
        unsigned __int64 nMarkTest2=0ULL;
        int nTestLinePurple;
        int nPurple=0;

        //Calcul mask blue/red
        for (nPosition=0;nPosition<m_nLengthTestPurple;nPosition++)
        {
                nMarkTest1=nMarkTest1<<1;
                nTestLinePurple=DetectLineBlue(m_pnLineCurent288[nPosition+m_nPositionTestPurple]);
                if (nTestLinePurple)
                        nPurple++;
                nMarkTest1|=nTestLinePurple;

                nMarkTest2=nMarkTest2<<1;
                nTestLinePurple=DetectLineRed(m_pnLineCurent288[nPosition+m_nPositionTestPurple]);
                if (nTestLinePurple)
                        nPurple++;
                nMarkTest2|=nTestLinePurple;

        }

        if (nPurple<20)
                return false;   //not enouth purple lines


        if (!m_bPhase)
        {
                nMarkTest1=nMarkTest1&0x5555555555555555LL;
                nMarkTest2=nMarkTest2&0xAAAAAAAAAAAAAAAALL;
        }
        else
        {
                nMarkTest1=nMarkTest1&0xAAAAAAAAAAAAAAAALL;
                nMarkTest2=nMarkTest2&0x5555555555555555LL;
        }


        m_nMarkPurple=nMarkTest1|nMarkTest2;

        return true;
}


bool CLaVache::FindKeyWithMarkChromi()
{
        if (m_bKeyFound)
                return false;   //already found

        //look for m_pnArrayMaskChromi[] with m_nMarkChromi
        int nBoundLow=0;
        int nBoundHigh=128*256-1;
        int nBoundMiddle=(nBoundHigh+nBoundLow)/2;

        if ((m_nMarkChromi>m_pnArrayMaskChromi[nBoundLow])
                ||(m_nMarkChromi<m_pnArrayMaskChromi[nBoundHigh]))
                return false;

        while(nBoundLow+1<nBoundHigh)
        {
                if (m_pnArrayMaskChromi[nBoundMiddle]==m_nMarkChromi)
                        break;

                if (m_pnArrayMaskChromi[nBoundMiddle]<m_nMarkChromi)
                        nBoundHigh=nBoundMiddle;
                else
                        nBoundLow=nBoundMiddle;

                nBoundMiddle=(nBoundHigh+nBoundLow)/2;
        }


        //set m_nKeyFound with m_psKeyMaskChromi
        if (m_pnArrayMaskChromi[nBoundMiddle]==m_nMarkChromi)
        {       //it's found -> magik !
                m_nKeyFound=m_psKeyMaskChromi[nBoundMiddle];
                m_bKeyFound=true;

                CreateLineOrder(m_nKeyFound,m_psSortLineCurrent);

                return true;
        }

        return false;
}



bool CLaVache::FindKeyWithMarkPurple()
{
        if (m_bKeyFound)
                return false;   //already found

        //look for m_pnArrayMaskPurple[] with m_nMarkPurple
        int nBoundLow=0;
        int nBoundHigh=128*256-1;
        int nBoundMiddle=(nBoundHigh+nBoundLow)/2;

        if ((m_nMarkPurple>m_pnArrayMaskPurple[nBoundLow])
                ||(m_nMarkPurple<m_pnArrayMaskPurple[nBoundHigh]))
                return false;

        while(nBoundLow+1<nBoundHigh)
        {
                if (m_pnArrayMaskPurple[nBoundMiddle]==m_nMarkPurple)
                        break;

                if (m_pnArrayMaskPurple[nBoundMiddle]<m_nMarkPurple)
                        nBoundHigh=nBoundMiddle;
                else
                        nBoundLow=nBoundMiddle;

                nBoundMiddle=(nBoundHigh+nBoundLow)/2;

        }

        //set m_nKeyFound with m_psKeyMaskPurple
        if (m_pnArrayMaskPurple[nBoundMiddle]==m_nMarkPurple)
        {       //it's found -> magik !
                m_nKeyFound=m_psKeyMaskPurple[nBoundMiddle];
                m_bKeyFound=true;

                CreateLineOrder(m_nKeyFound,m_psSortLineCurrent);

                return true;
        }


        return false;
}


bool CLaVache::LoadFileChromi()
{
        FILE *sFile = fopen(File_ChromiKey, "rb");

        if(sFile == NULL)
      return false;

        if (!CheckFileID(sFile))
        {
                fclose(sFile);
                return false;
        }

        // m_nLengthTestChromi
        fread(&m_nLengthTestChromi, sizeof(m_nLengthTestChromi),1,sFile);   

        // m_nPositionTestChromi
        fread(&m_nPositionTestChromi, sizeof(m_nPositionTestChromi),1,sFile);   
   
        for (int nKeyTested=0;nKeyTested<128*256;nKeyTested++)
        {
           // m_pnArrayMaskChromi
           fread(m_pnArrayMaskChromi+nKeyTested, sizeof(*m_pnArrayMaskChromi),1,sFile);

           // m_psKeyMaskChromi
           fread(m_psKeyMaskChromi+nKeyTested, sizeof(*m_psKeyMaskChromi),1,sFile);
        }


        fclose(sFile);

        if (m_nLengthTestChromi!=64)
                return false;

        if ((m_nPositionTestChromi+m_nLengthTestChromi>CURRENTY)
                ||(m_nPositionTestChromi<0))
                return false;

        return true;
}


bool CLaVache::LoadFilePurple()
{
        FILE *sFile = fopen(File_PurpleKey, "rb");

        if(sFile == NULL)
      return false;

        if (!CheckFileID(sFile))
        {
                fclose(sFile);
                return false;
        }

        // m_nLengthTestPurple
        fread(&m_nLengthTestPurple, sizeof(m_nLengthTestPurple),1,sFile);   

        // m_nPositionTestPurple
        fread(&m_nPositionTestPurple, sizeof(m_nPositionTestPurple),1,sFile);   
   
        for (int cleftest=0;cleftest<128*256;cleftest++)
        {
           // m_pnArrayMaskPurple
           fread(m_pnArrayMaskPurple+cleftest, sizeof(*m_pnArrayMaskPurple),1,sFile);

           // ClefEmpreinteViolet
           fread(m_psKeyMaskPurple+cleftest, sizeof(*m_psKeyMaskPurple),1,sFile);
        }

        fclose(sFile);

        if (m_nLengthTestPurple!=64)
                return false;

        if ((m_nPositionTestPurple+m_nLengthTestPurple>CURRENTY)
                ||(m_nPositionTestPurple<0))
                return false;

        return true;
}


//debug
__int64 CLaVache::GetRDTSCTime() 
{
    union 
        {
          __int64 ll;
          struct { int lo;int hi ;} s;
        } time;

          __asm__ __volatile__ (
                "cpuid                       \n\t"
                "rdtsc                       \n\t"
                "movl %%eax, %0              \n\t"
                "movl %%edx, %1              \n\t"

           : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

           :  "m" (time.s.lo), "m" (time.s.hi)

           : "%eax", "%edx", "memory"
        );

        return time.ll;
}


bool CLaVache::UpdateInputLine(short ** ppsInputBuffer,BOOL bIsOdd)
{
        int py;

        m_bIsOdd=bIsOdd;
        m_bKeyFound=false;

        //get actual line
        short ** ppsInBuf=ppsInputBuffer;
        if ((m_bUsedLoadImage)&&(m_pnTamponChargeCapture))
        {
                short * psLine[CURRENTY];
                for (py=0;py<CURRENTY;py++)
                        psLine[py]=(short*)m_pnTamponChargeCapture+(py*2+(1-bIsOdd))*m_nSizeBufferX;
                ppsInBuf=psLine;
        }

        //fisrt 32
        for (py=0;py<OFFSET32;py++)
                m_pnLineCurent288[py]=m_pnLineNext32[py];

        //second 32->287
        for (py=0;py<CURRENTY-OFFSET32;py++)
                m_pnLineCurent288[py+OFFSET32]=(int*)ppsInBuf[py];
        
        //copy next line buffer
        for (py=0;py<OFFSET32;py++)
                m_pnLineNext32[py]=(int*)ppsInBuf[CURRENTY-OFFSET32+py-1];

        if (!m_pnLineCurent288[0])
                return false;
        
        return true;
}


void CLaVache::UpdateOutputLine(short ** ppsOutputBuffer)
{
        for (int py=0;py<CURRENTY;py++)
        {       m_pnLineSort288[py]=m_pnLineCurent288[m_psSortLineCurrent[py]];
                ppsOutputBuffer[py]=(short*)m_pnLineSort288[py];
        }
}


void CLaVache::UpdateOldLine()
{

        m_bOldKeyFound[m_bIsOdd]=m_bKeyFound;

        for (int py=0;py<CURRENTY;py++)
                m_pnLineOldSort288[m_bIsOdd][py]=m_pnLineSort288[py];

        for (int i=0;i<NCOLUMNTESTED;i++)
                m_pnColumnOldSelect[m_bIsOdd][i]=m_pnColumnSelect[i];
}


bool CLaVache::LoadFileKey()
{
   
   int   nZlr;
   int   nInputValue;
    char key_file[255];

   sprintf(key_file, "%s/%s", home_dir, "key.txt");
   FILE *sFile = fopen(key_file, "r");

   if(sFile == NULL)
      return false;

   for(nZlr=0; nZlr<256; nZlr++)
   {
      if(fscanf(sFile, "%d\n", &nInputValue) == EOF)
         return false;

      m_cKeyLine[nZlr] = (char)(nInputValue&0x1f);
   }

   fclose(sFile);
   return true;
}


 void CLaVache::CreateLineOrder(int nKey, short * psSort)
{

        int i;
        BYTE j;
        short buffer[32];
        int offset;
        int increment;

        if ((nKey<0)||(nKey>=128*256))
                nKey=0;

        offset=(nKey>>7);
        increment=((nKey&0x7f)*2+1);

        FillMemory(buffer,sizeof(buffer),0xff); //-1

        j = (BYTE)offset;

        for (i=0;i<255;i++)
        {
                if (buffer[m_cKeyLine[j]] == -1)
                        psSort[i]=m_cKeyLine[j];
                else
                        psSort[i]=buffer[m_cKeyLine[j]];

                buffer [m_cKeyLine[j]] = (short)(32+i);
                j += increment;                                 // Note that j is a BYTE
        }


        CopyMemory(psSort+255,buffer,sizeof(buffer));
        psSort[287]=287;

}


bool CLaVache::FindKey(short ** ppsInputBuffer,short ** ppsOutputBuffer,BOOL bIsOdd,bool bPhaseFrame)
{
        if (!UpdateInputLine(ppsInputBuffer,(bIsOdd!=0)))
                return m_bKeyFound;
        //1
        UseKeyRepeat();
        //2
        UpdatePhase();

        if (m_bFilterPurple)
        {       //BT878
                if(UpdateMarkPurple())
                        FindKeyWithMarkPurple();
        }
        else
        {       //BT848
                if (UpdateMarkChromi())
                        FindKeyWithMarkChromi();
        }

        //3
        FindKeyWithDiff();

        RecordKeyRepeat(m_bKeyFound,m_nKeyFound);

        UpdateOutputLine(ppsOutputBuffer);

        //filter
        if (m_bKeyFound)
                FilterFrame(bPhaseFrame);

        UpdateOldLine();

        return (m_bKeyFound);
}


bool CLaVache::LoadFilePossibleNormal()
{

        FILE *sFile = fopen(File_PossibleNormal, "rb");

        if(sFile == NULL)
                return false;

        if (!CheckFileID(sFile))
        {
                fclose(sFile);
                return false;
        }

        fread(m_pbPossibleNormal, sizeof(*m_pbPossibleNormal),CURRENTY*CURRENTY,sFile);

        fclose(sFile);

        return true;
}


bool CLaVache::LoadFileLineNormal()
{
   
        int   nZlr;
        FILE *sFile = fopen(File_LineNormal, "rb");
        int nLineIndex;
        int nFullSize;
        short * adresse_clef;

        if(sFile == NULL)
                return false;

        if (!CheckFileID(sFile))
        {
                fclose(sFile);
                return false;
        }

        //number of lines
        fread(&m_nTestNormalMaxi, sizeof(m_nTestNormalMaxi),1,sFile);

        //total size
        fread(&nFullSize, sizeof(nFullSize),1,sFile);

        m_psCountKeyLineNormal=new short[nFullSize];
        if(m_psCountKeyLineNormal == NULL)
        {
                OutputDebugString("IDS_ERR_MEMORY");
                return false;
        }

        m_pnPositionLineNormal=new int[m_nTestNormalMaxi];
        if(m_pnPositionLineNormal == NULL)
        {
                OutputDebugString("IDS_ERR_MEMORY");
                return false;
        }

        m_ppsAdressKeyLineNormal=new short* [m_nTestNormalMaxi*CURRENTY*CURRENTY+1];
        if(m_ppsAdressKeyLineNormal == NULL)
        {
                OutputDebugString("IDS_ERR_MEMORY");
                return false;
        }


        m_pcKeyLineNormal=new unsigned char[m_nTestNormalMaxi*CURRENTY*CURRENTY];
        if(m_pcKeyLineNormal == NULL)
        {
                OutputDebugString("IDS_ERR_MEMORY");
                return false;
        }

        //load key values (dynamic array)
        fread(m_psCountKeyLineNormal, sizeof(*m_psCountKeyLineNormal),nFullSize,sFile);

        adresse_clef=m_psCountKeyLineNormal;
        for (nLineIndex=0;nLineIndex<m_nTestNormalMaxi;nLineIndex++)
        {
                // record the number of line
                fread(m_pnPositionLineNormal+nLineIndex, sizeof(*m_pnPositionLineNormal),1,sFile);

                //record numbe of key, line per line
                fread(m_pcKeyLineNormal+nLineIndex*CURRENTY*CURRENTY, sizeof(*m_pcKeyLineNormal),CURRENTY*CURRENTY,sFile);

                for(nZlr=0; nZlr<CURRENTY*CURRENTY; nZlr++)
                {
                        //calcul pointer
                        m_ppsAdressKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nZlr]=adresse_clef;
                        adresse_clef+=*(m_pcKeyLineNormal+nLineIndex*CURRENTY*CURRENTY+nZlr);
                }

        }

        fclose(sFile);
        return true;
}




bool CLaVache::FindKeyNormal(int nLineIndex)
{
        int i;
        int nTopNearLineNear;
        int nBottomNearLineNear;
        int nPropaGoodKey;
        int nCurrentColumnTest;
        int nKeyTested,nIndexKeyTested;
        int nBestScore;
        int nKeyKeep=0;
        int nIndexTested=0;
        int nFlipLine;

        int nOldBadKey=m_nKeyFound;

        m_nTypeTryPrevious=0;

        //FindLineNormal
        if (!FindLineNormal(m_pnPositionLineNormal[nLineIndex],&nBottomNearLineNear,&nTopNearLineNear))
                return(false);          //no lines found
        
        if ((m_pcKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nBottomNearLineNear*CURRENTY+nTopNearLineNear]==0)
                &&(m_pcKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nTopNearLineNear*CURRENTY+nBottomNearLineNear]==0))
                return(false);          //no lines found

        // calcul m_pnDiffColumnNormal ?
        if (m_pnDiffColumnNormal[0]==-1)
        {       //m_pnDiffColumnNormal haven't yet calculed
                for (int py=0;py<CURRENTY;py++) 
                        m_psSortLineCurrent[py]=py;

                //calcul nPropaGoodKey
                nPropaGoodKey=0;
                for (i=0;i<NCOLUMNTESTED;i++)
                {
                        nPropaGoodKey+=CalculDiffColumn(i);
                        m_pnDiffColumnNormal[i]=nPropaGoodKey;
                }

        }

        //choose nCurrentColumnTest with m_nTestDone
        nCurrentColumnTest=m_nTestDone/2+3;
        if (nCurrentColumnTest>NCOLUMNTESTED)
                nCurrentColumnTest=NCOLUMNTESTED;

        // CalculDiffColumn
        nBestScore=0x0FFFFFFF;
        for (nIndexKeyTested=0;nIndexKeyTested<m_pcKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nBottomNearLineNear*CURRENTY+nTopNearLineNear];nIndexKeyTested++)
        {
                nKeyTested=*(m_ppsAdressKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nBottomNearLineNear*CURRENTY+nTopNearLineNear]+nIndexKeyTested);
                CreateLineOrder(nKeyTested,m_psSortLineCurrent);

                nPropaGoodKey=0;
                for (i=0;i<nCurrentColumnTest;i++)      
                {
                        nPropaGoodKey+=CalculDiffColumn(i);

                        if (nPropaGoodKey>nBestScore)
                                break;          // nPropaGoodKey is over nBestScore

                }
                
                nIndexTested++;

                if (nPropaGoodKey<nBestScore)
                {
                        nBestScore=nPropaGoodKey;
                        nKeyKeep=nKeyTested;
                }

        }

        //validation ?
        if (nIndexTested>0)
        {
                m_nRateGoodProba=(m_pnDiffColumnNormal[nCurrentColumnTest-1]-nBestScore)*100/(nBestScore+1);
                if (m_nRateGoodProba<0)
                        m_nRateGoodProba=0;
                
                m_nKeyFound=(short)nKeyKeep;

                if (m_nRateGoodProba>(m_nProbaNormalDynam+80-m_pcKeyAlreadyFound[nKeyKeep]*35)) 
                {

                        CreateLineOrder(nKeyKeep,m_psSortLineCurrent);
                        m_bKeyFound=true;
                        m_nTestDone++;

                        __asm__  __volatile__  ("emms");                              // output MMX mode (used in CalculDiffColumn)

                        //limit
                        m_nProbaNormalLimitSup=(m_nProbaNormalLimitSup*COEFFILTERLIMIT+m_nRateGoodProba*COEFFILTERLIMIT)/(COEFFILTERLIMIT+1);
                        if (m_nProbaNormalLimitSup>TresholdNormalLimitSup)
                                m_nProbaNormalLimitSup=TresholdNormalLimitSup;

                        return(true);
                }

        }


        
        //flip nBottomNearLineNear & nTopNearLineNear
        nFlipLine=nBottomNearLineNear;
        nBottomNearLineNear=nTopNearLineNear;
        nTopNearLineNear=nFlipLine;

        // CalculDiffColumn
        for (nIndexKeyTested=0;nIndexKeyTested<m_pcKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nBottomNearLineNear*CURRENTY+nTopNearLineNear];nIndexKeyTested++)
        {
                nKeyTested=*(m_ppsAdressKeyLineNormal[nLineIndex*CURRENTY*CURRENTY+nBottomNearLineNear*CURRENTY+nTopNearLineNear]+nIndexKeyTested);
                CreateLineOrder(nKeyTested,m_psSortLineCurrent);

                nPropaGoodKey=0;
                for (i=0;i<nCurrentColumnTest;i++)      
                {
                        nPropaGoodKey+=CalculDiffColumn(i);

                        if (nPropaGoodKey>nBestScore)
                                break;                  // nPropaGoodKey is over nBestScore
                }


                nIndexTested++;


                if (nPropaGoodKey<nBestScore)
                {
                        nBestScore=nPropaGoodKey;
                        nKeyKeep=nKeyTested;
                }


        }
        
        __asm__ __volatile__ ("emms");                              // output MMX mode (used in CalculDiffColumn)


        //validation ?
        if (nIndexTested>0)
        {
                m_nRateGoodProba=(m_pnDiffColumnNormal[nCurrentColumnTest-1]-nBestScore)*100/(nBestScore+1);
                if (m_nRateGoodProba<0)
                        m_nRateGoodProba=0;

                m_nKeyFound=(short)nKeyKeep;

                if (m_nRateGoodProba>m_nProbaNormalDynam+20-m_pcKeyAlreadyFound[nKeyKeep]*40)           //film
                {
                        CreateLineOrder(nKeyKeep,m_psSortLineCurrent);

                        m_bKeyFound=true;
                        m_nTestDone++;

                        //limit
                        m_nProbaNormalLimitSup=(m_nProbaNormalLimitSup*COEFFILTERLIMIT+m_nRateGoodProba*COEFFILTERLIMIT)/(COEFFILTERLIMIT+1);
                        if (m_nProbaNormalLimitSup>TresholdNormalLimitSup)
                                m_nProbaNormalLimitSup=TresholdNormalLimitSup;

                        return(true);
                }

        }

        //limit
        if ((m_nTestDone>0)&&(nOldBadKey!=m_nKeyFound)&&(m_pcKeyAlreadyFound[nKeyKeep]==0))
        {
                m_nProbaNormalLimitInf=(m_nProbaNormalLimitInf*COEFFILTERLIMIT+m_nRateGoodProba*COEFFILTERLIMIT)/(COEFFILTERLIMIT+1);
                if (m_nProbaNormalLimitInf>TresholdNormalLimitInf)
                        m_nProbaNormalLimitInf=TresholdNormalLimitInf;
        }

        //incrise m_pcKeyAlreadyFound[nKeyKeep]
        m_pcKeyAlreadyFound[nKeyKeep]++;
        m_nTestDone++;

        return(false); 
}



 bool CLaVache::FindLineNormal(int nLine,int * nBottomNearLineNear,int * nTopNearLineNear)
{ //trouve les lignes les plus ressemblantes dans l'image courante crypte (ligne du dessus et du dessous)
        int py;
        int nBestScore,nBestLine_dessus,nBestLine_dessous;

        nBestLine_dessus=0;
        nBestLine_dessous=0;
        nBestScore=0x0FFFFFFF;
        
        for (py=0;py<CURRENTY;py+=1)   
        {

                if (m_pbPossibleNormal[nLine+py*CURRENTY])
                {
                        if (m_pnMemoDistance[py*CURRENTY+nLine]==-1)
                        {
                                CalculDiffLine(nLine,py);       //calcul distance
                        }

                                //find the min
                        if ((m_pnMemoDistance[py*CURRENTY+nLine]!=-1)&&(m_pnMemoDistance[py*CURRENTY+nLine]<nBestScore))
                        {
                                nBestScore=m_pnMemoDistance[py*CURRENTY+nLine];
                                nBestLine_dessus=py;

                        }
                
                }
                
        }


        if (nBestScore==0x0FFFFFFF)
                return false;


        nBestScore=0x0FFFFFFF;
        
        for (py=0;py<CURRENTY;py+=1) 
        {
                if ((m_pbPossibleNormal[nLine*CURRENTY+py])&&(py!=nBestLine_dessus))
                {
                        if (m_pnMemoDistance[py*CURRENTY+nLine]==-1)
                        {
                                CalculDiffLine(nLine,py);       //calcul distance
                        }
                        // else { int yy=0; }
                                
                        //find the min
                        if ((m_pnMemoDistance[py*CURRENTY+nLine]!=-1)&&(m_pnMemoDistance[py*CURRENTY+nLine]<nBestScore))
                        {
                                nBestScore=m_pnMemoDistance[py*CURRENTY+nLine];
                                nBestLine_dessous=py;
                        } 
                }
                
        }

        __asm__ __volatile__ ("emms");                              // output MMX mode

        if (nBestScore==0x0FFFFFFF)
                return false;

        *nBottomNearLineNear=nBestLine_dessous;
        *nTopNearLineNear=nBestLine_dessus;


        return(true);
}

 void CLaVache::CalculDiffLine(int ligne1,int ligne2)
{
       // const __int64 maskdw  = 0x00000000FFFFFFFFLL;
       // const __int64 maskw12 = 0x0000FFFF0000FFFFLL;

        int * pnLine1,* pnLine2;
        int nSumDiff;
        int nLoopPx;

        if (m_pnMemoDistance[ligne1*CURRENTY+ligne2]!=-1)       //calcul yet calcul
                return; 

        nSumDiff=0;

        pnLine1=m_pnLineCurent288[ligne1];
        pnLine2=m_pnLineCurent288[ligne2];

        nLoopPx=m_nSizeBufferX*sizeof(short)/sizeof(__int64);
        __int64 masklum = 0x00FF00FF00FF00FFLL;

          __asm__ __volatile__ (

                "movl            %0, %%edi \n\t"
                "pxor    %%mm0, %%mm0        \n\t" // mm0=0

                "movl            %1, %%esi \n\t"
                "movq    %4, %%mm7      \n\t" //mm7=masklum

                "movl            %2, %%ecx       \n\t"
                "movq    %%mm7, %%mm6        \n\t" //mm6=masklum

//                "jmp             CDL.AutrePixel  \n\t"
                "jmp 0f \n\t"
                ".align 16                   \n\t"
//         "CDL.AutrePixel:                 \n\t"
                "0: \n\f"

                "movq    (%%edi), %%mm1      \n\t"      //mm1=*pnLine1
                "movq    (%%esi), %%mm2      \n\t"      //mm2=*pnLine2

                "pand    %%mm7, %%mm1        \n\t"              //mm1'=mm1&masklum
                "pand    %%mm6, %%mm2        \n\t"              //mm2'=mm2&masklum

                "movq    %%mm1, %%mm3        \n\t"              //mm3=mm1
                "movq    %%mm2, %%mm4        \n\t"              //mm4=mm2

                "psubusw %%mm2, %%mm1        \n\t"              //mm1=mm1-mm2 -> =0 si ngatif
                "addl            $8, %%edi   \n\t"

                "psubusw %%mm3, %%mm4        \n\t"              //mm4=mm2-mm1 -> =0 si ngatif
                "addl            $8, %%esi   \n\t"

                "por             %%mm4, %%mm1 \n\t"                     //mm1=mm1|mm4 -> mm1=|mm1'-mm2'|
                "decl            %%ecx       \n\t"

                "paddw   %%mm1, %%mm0        \n\t"
//                "jnz         CDL.AutrePixel  \n\t"
                "jnz 0b \n\t"

                //additionne mm0
                "movq            %%mm0, %%mm1 \n\t"             //mm1=mm0= A B C D (w)
                "pxor            %%mm2, %%mm2 \n\t"             //mm2    = 0 0 0 0 (w)

                "punpcklwd       %%mm2, %%mm0 \n\t"             //mm0=     0 C 0 D (w)
                "punpckhwd       %%mm2, %%mm1 \n\t"             //mm1=     0 A 0 B (w)

                "paddd           %%mm1, %%mm0 \n\t"             //mm0= (A+C)  (B+D) (dw)

                "movq            %%mm0, %%mm1 \n\t"             //mm1= (A+C)  (B+D) (dw)
                "punpckhdq       %%mm1, %%mm1 \n\t"             //mm1= (A+C)  (A+C) (dw)
                "paddd           %%mm1, %%mm0 \n\t"             //mm0=   X    (A+B+C+D) (dw)

                "movd            %%mm0,%3       \n\t"


           : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

           : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
            "m" (pnLine1), "m" (pnLine2), "m" (nLoopPx), "m" (nSumDiff),"m" (masklum)

           : "%ecx", "%edi", "%esi" // CHECKASM: clobber list
        );

        m_pnMemoDistance[ligne1*CURRENTY+ligne2]=nSumDiff;
        m_pnMemoDistance[ligne2*CURRENTY+ligne1]=nSumDiff;


}



 int CLaVache::CalculDiffColumn(int nIndexCulumn)
{ 
         __int64 masklum = 0x00FF00FF00FF00FFLL;
        int nSumDiffVabs = 0;

        int *pValue=&(m_pnColumnValue[nIndexCulumn][0]);
        short * psSort=&m_psSortLineCurrent[284];

          __asm__ __volatile__ (
                "movl                    %0, %%esi \n\t"
                "movq                    %3, %%mm6 \n\t"

                "movl                    %1, %%edi \n\t"
                "pxor            %%mm7, %%mm7 \n\t"                     // mm7 = nSumDiffVabs = 0

                "movl                    $284, %%ecx \n\t"                      //py=284        (multiple de 4)
                "xorl                    %%eax, %%eax \n\t"

                "movw                    2(%%edi), %%ax \n\t"
                "movd            (%%esi,%%eax,4), %%mm3 \n\t"           //mm3 = 00000000 00000000 00000000 00000000  VVVVVVVV Y6Y6Y6Y6 UUUUUUUU Y5Y5Y5Y5

//                "jmp                     CDC.Boucle1 \n\t"
                "jmp 0f \n\t"
                ".align 16                   \n\t"
//               "CDC.Boucle1:\n\t"
                "0: \n\t"
                //read

                "movw                    (%%edi), %%ax \n\t"
                "movd            (%%esi,%%eax,4), %%mm0 \n\t"                   //mm0 = 00000000 00000000 00000000 00000000  VVVVVVVV Y2Y2Y2Y2 UUUUUUUU Y1Y1Y1Y1

                "movw                    -2(%%edi), %%ax \n\t"
                "movd            (%%esi,%%eax,4), %%mm1 \n\t"                   //mm1 = 00000000 00000000 00000000 00000000  VVVVVVVV Y4Y4Y4Y4 UUUUUUUU Y3Y3Y3Y3

                "punpckldq       %%mm0, %%mm0 \n\t"                     //mm0 = VVVVVVVV Y2Y2Y2Y2 UUUUUUUU Y1Y1Y1Y1  VVVVVVVV Y2Y2Y2Y2 UUUUUUUU Y1Y1Y1Y1
                "punpckldq       %%mm3, %%mm1 \n\t"                     //mm1 = VVVVVVVV Y6Y6Y6Y6 UUUUUUUU Y5Y5Y5Y5  VVVVVVVV Y4Y4Y4Y4 UUUUUUUU Y3Y3Y3Y3

                "pand            %%mm6, %%mm0 \n\t"                     //mm0 = 00000000 Y2Y2Y2Y2 00000000 Y1Y1Y1Y1  00000000 Y2Y2Y2Y2 00000000 Y1Y1Y1Y1
                "movq            %%mm1, %%mm3 \n\t"                     //mm3 = VVVVVVVV Y6Y6Y6Y6 UUUUUUUU Y5Y5Y5Y5  VVVVVVVV Y4Y4Y4Y4 UUUUUUUU Y3Y3Y3Y3

                "movq            %%mm0, %%mm2 \n\t"                     //mm2 = 00000000 Y2Y2Y2Y2 00000000 Y1Y1Y1Y1  00000000 Y2Y2Y2Y2 00000000 Y1Y1Y1Y1
                "pand            %%mm6, %%mm1 \n\t"                     //mm1 = 00000000 Y6Y6Y6Y6 00000000 Y5Y5Y5Y5  00000000 Y4Y4Y4Y4 00000000 Y3Y3Y3Y3

                "movq            %%mm1, %%mm4 \n\t"                     //mm4 = 00000000 Y6Y6Y6Y6 00000000 Y5Y5Y5Y5  00000000 Y4Y4Y4Y4 00000000 Y3Y3Y3Y3
                //|mm1'-mm2'|
                "psubusw         %%mm1, %%mm0 \n\t"                     //mm0 =  mm0-mm1 -> =0 si ngatif (w)

                "psubusw         %%mm2, %%mm4 \n\t"                     //mm4 = mm4-mm2 -> =0 si ngatif (w)
                "subl                    $4, %%edi \n\t"                        //edi-=4

                "por                     %%mm4, %%mm0 \n\t"                     //mm0 = mm0|mm4 -> mm1=|mm1'-mm2'| (w)
                //add the result into mm7

                "subl                    $2, %%ecx \n\t"                        //py-=2
                "paddw           %%mm0, %%mm7 \n\t"                     //mm7 += mm0

//                "jnz                     CDC.Boucle1 \n\t"
                "jnz 0b \n\t"

                //add mm7
                "movq            %%mm7, %%mm1 \n\t"             //mm1=mm7= A B C D (w)
                "pxor            %%mm2, %%mm2 \n\t"             //mm2    = 0 0 0 0 (w)

                "punpcklwd       %%mm2, %%mm7 \n\t"             //mm7=     0 C 0 D (w)
                "punpckhwd       %%mm2, %%mm1 \n\t"             //mm1=     0 A 0 B (w)

                "paddd           %%mm1, %%mm7 \n\t"             //mm7= (A+C)  (B+D) (dw)

                "movq            %%mm7, %%mm1 \n\t"             //mm1= (A+C)  (B+D) (dw)
                "punpckhdq       %%mm1, %%mm1 \n\t"             //mm1= (A+C)  (A+C) (dw)
                "paddd           %%mm1, %%mm7 \n\t"             //mm7=   X    (A+B+C+D) (dw)

                "movd            %%mm7, %2 \n\t"

                "emms \n\t"

           : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

           : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
            "m" (pValue), "m" (psSort), "m" (nSumDiffVabs) , "m" (masklum)

           : "%eax", "%ecx", "%edi", "%esi", "memory" // CHECKASM: clobber list
        );


        return(nSumDiffVabs);
}




//main process
bool CLaVache::FindKeyWithDiff()
{       
        int i;

        m_nRateGoodProba=0;     

        if ((!m_bKeyFound)
                &&(m_pnLineCurent288[0])
                &&(m_pnLineOldSort288[m_bIsOdd][0]))
        {
                //init values
                m_nTestDone=0;
                ZeroMemory(m_pcKeyAlreadyFound,(128*256+1)*sizeof(*m_pcKeyAlreadyFound));

                if (m_nProbaNormalDynam<TresholdNormalDynamicInf)
                        m_nProbaNormalDynam=TresholdNormalDynamicInf;

                //calcul  mask colums
                CalculColumnMask(NCOLUMNTESTED); 

                if ((!m_bOldKeyFound[m_bIsOdd])||(m_nTypeTryPrevious==1)||(!IsFrameFixe()))
                //if (1==1)             ///debug
                {
                        //FindKeyNormal
                        FillMemory(m_pnMemoDistance,CURRENTY*CURRENTY*sizeof(int),0xFF); //-1 
                        m_pnDiffColumnNormal[0]=-1;     

                        CalculInterPixelLine(); 

                        for (i=0;i<m_nTestNormalUser;i++)
                        {
                                //find key with normal mode and line n m_psInterPixel[i]
                                if (FindKeyNormal(m_psInterPixelNormal[i]))
                                        break;          //ok key found
                        }

                        //calcul Dynam coeff normal
                        m_nProbaNormalDynam=(m_nProbaNormalDynam*2 + (abs(m_nProbaNormalLimitSup-m_nProbaNormalLimitInf)/2+m_nProbaNormalLimitInf)/COEFFILTERLIMIT )/3;

                }
                else
                {
                        if (m_nProbaPreviousDynam<TresholdPreviousDynamicInf)
                                m_nProbaPreviousDynam=TresholdPreviousDynamicInf;

                        int nTestPreviousDone=0;
                        int nTestNormalDone=0;

                        while ((nTestPreviousDone<m_nTestPreviousUser)
                                ||(nTestNormalDone<m_nTestNormalUser))
                        {
                                
                                if (nTestPreviousDone==0)
                                {
                                        SetColumnOldSelect();
                                        CalculInterPixelLinePrevious();
                                        m_pnDiffColumnPrevious[0]=-1;   
                                }

                                if (nTestPreviousDone<m_nTestPreviousUser)
                                {
                                        if (FindKeyPrevious(m_psInterPixelPrevious[nTestPreviousDone]))
                                                break;          //ok key found
                                        nTestPreviousDone++;
                                }
                                if (nTestPreviousDone<m_nTestPreviousUser)
                                {
                                        if (FindKeyPrevious(m_psInterPixelPrevious[nTestPreviousDone]))
                                                break;          //ok key found
                                        nTestPreviousDone++;
                                }

                                if (nTestNormalDone==0)
                                {
                                        FillMemory(m_pnMemoDistance,CURRENTY*CURRENTY*sizeof(int),0xFF); //-1 
                                        m_pnDiffColumnNormal[0]=-1;     
                                        CalculInterPixelLine(); 
                                }

                                if (nTestNormalDone<m_nTestNormalUser)
                                {
                                        if (FindKeyNormal(m_psInterPixelNormal[nTestNormalDone]))
                                                break;          //ok key found
                                        nTestNormalDone++;
                                }
                                if (nTestNormalDone<m_nTestNormalUser)
                                {
                                        if (FindKeyNormal(m_psInterPixelNormal[nTestNormalDone]))
                                                break;          //ok key found
                                        nTestNormalDone++;
                                }


                        }

                        //calcul Dynam coeff previous
                        m_nProbaPreviousDynam=(m_nProbaPreviousDynam*2 + (abs(m_nProbaPreviousLimitSup-m_nProbaPreviousLimitInf)/2+m_nProbaPreviousLimitInf)/COEFFILTERLIMIT )/3;
                        //calcul Dynam coeff normal
                        m_nProbaNormalDynam=(m_nProbaNormalDynam*2 + (abs(m_nProbaNormalLimitSup-m_nProbaNormalLimitInf)/2+m_nProbaNormalLimitInf)/COEFFILTERLIMIT )/3;

                }


                //FindHeightBlack
                FindHeightBlackBorder();

                //if ((m_bKeyFound)&&(m_nTypeTryPrevious==2))
                //      ShowColumn();           //debug
        }
        
        return (m_bKeyFound);
} 


 void CLaVache::CalculColumnMask(int nb_colonne)
{
        int i,py;
        int nColumnUsed;
        int nMaxScramb;
        int nMaxPosition;
        int nTemp;
        int nBound1,nBound2;
        bool bMaskColumn[MAXIMCOLUMNTESTED];   
        int nScrambColumn[MAXIMCOLUMNTESTED];   
        int nColumnPos[MAXIMCOLUMNTESTED];
        int nAverageColumn[MAXIMCOLUMNTESTED];

        ZeroMemory(nScrambColumn,sizeof(nScrambColumn));
        ZeroMemory(nAverageColumn,sizeof(nAverageColumn));

        int c =m_nSizeBufferX*1024/(2*(MAXIMCOLUMNTESTED+1));

        for (i=0;i<MAXIMCOLUMNTESTED;i++)
                nColumnPos[i]=(c*(i+1))/1024;  

        for (py=0;py<CURRENTY;py+=2)
        {
                for (i=0;i<MAXIMCOLUMNTESTED;i++)
                {
                        nAverageColumn[i]+=(unsigned char)(*(m_pnLineCurent288[py]+nColumnPos[i]));
                }
        }
        for (i=0;i<MAXIMCOLUMNTESTED;i++)
        {
                nAverageColumn[i]=nAverageColumn[i]/(CURRENTY/2);
        }

        for (py=0;py<CURRENTY-1;py+=2)
        {
                for (i=0;i<MAXIMCOLUMNTESTED;i++)
                {
                        nTemp=(unsigned char)(*(m_pnLineCurent288[py]+nColumnPos[i]))-nAverageColumn[i];
                        if (nTemp<0)
                                nTemp=0-nTemp;
                        nScrambColumn[i]+=nTemp;
                }
        }


        nColumnUsed=0;
        
        ZeroMemory(bMaskColumn,sizeof(bMaskColumn));

        
        while(nColumnUsed<nb_colonne)
        {
                nMaxScramb=0;
                nMaxPosition=1;
                //search max
                nBound1=0;
                nBound2=MAXIMCOLUMNTESTED;
                
                //limit nBounds
                if (nColumnUsed==0)
                        nBound2=MAXIMCOLUMNTESTED/2;
                if (nColumnUsed==1)
                        nBound1=MAXIMCOLUMNTESTED/2;

                for (i=nBound1;i<nBound2;i+=1)
                {
                        if (!bMaskColumn[i])
                        {

                                if (nScrambColumn[i]>=nMaxScramb)
                                {
                                        nMaxScramb=nScrambColumn[i];
                                        nMaxPosition=i;
                                }
                        }
                }
                
                m_pnColumnSelect[nColumnUsed]=short(c*(nMaxPosition+1)/1024);
                bMaskColumn[nMaxPosition]=true;
                nColumnUsed++;
        }

        SetColumnSelect();
}


void CLaVache::SetColumnSelect() 
{
        //Set value in m_pnColumnValue
        int * pnLine;
        
        for (int py=0;py<CURRENTY;py++)
        {
                pnLine=m_pnLineCurent288[py];
                
                for (int nColum=0;nColum<NCOLUMNTESTED;nColum++)
                {
                        m_pnColumnValue[nColum][py]=pnLine[m_pnColumnSelect[nColum]];

                }
        }

}


void CLaVache::SetColumnOldSelect() 
{
        //Set value in m_pnColumnValue
        int * pnLine;
        
        for (int py=0;py<CURRENTY;py++)
        {
                pnLine=m_pnLineOldSort288[m_bIsOdd][py];
                
                for (int nColum=0;nColum<NCOLUMNTESTED;nColum++)
                {
                        m_pnColumnOldSortValue[m_bIsOdd][nColum][py]=pnLine[m_pnColumnSelect[nColum]];
                }
        }

}


 void CLaVache::CalculInterPixelLine() 
{
        int i,j,py;
        int * pnLine;
        int nBestScore,position;
        int nSumDiff=0;

        int nLoopPx=m_nSizeBufferX/8-2;
        
         const __int64 mask0000ffff = 0x0000ffff0000ffffLL;
         const __int64 maskffff0000 = 0xffff0000ffff0000LL;
         const __int64 mask00ff00ff = 0x00ff00ff00ff00ffLL;


        for (py=0;py<m_nTestNormalMaxi;py++)
        {
                pnLine=m_pnLineCurent288[m_pnPositionLineNormal[py]];

                  __asm__ __volatile__ (
                        "pxor    %%mm0, %%mm0        \n\t"      // mm0=0

                        "movl         %0, %%esi \n\t"
                        "movq    %5, %%mm7 \n\t" //mm7=masklum
                        "movq    %%mm7, %%mm5        \n\t"      //mm5=masklum

                        "movl         %1,%%ecx       \n\t"
//                        "jmp CIPL.AutrePixel              \n\t"
                        "jmp 0f \n\t"

                        ".align 16                   \n\t"
//                 "CIPL.AutrePixel:                 \n\t"
                        "0: \n\t"

                        //1
                        "movq    (%%esi), %%mm1      \n\t"      //mm1=*pnLine
                        "movq    8(%%esi), %%mm2     \n\t"      //mm2=*pnLine +2 pixel


                        "pand    %%mm7, %%mm1        \n\t"              //mm1=mm1&masklum
                        "pand    %%mm5, %%mm2        \n\t"              //mm2=mm2&masklum

                        "movq    %%mm1, %%mm3        \n\t"              //mm3=mm1
                        "movq    %%mm2, %%mm4        \n\t"              //mm4=mm2

                        "psrld   $16, %%mm3          \n\t"              //mm3=mm3>>16   (d)
                        "pslld   $16, %%mm4          \n\t"              //mm4=mm4<<16   (d)

                        "pand    %3, %%mm1 \n\t"      //mm1=mm1&mask0000ffff
                        "pand    %4, %%mm2 \n\t"      //mm2=mm2&maskffff0000

                        "por             %%mm2, %%mm1 \n\t"                     //mm1|=mm2
                        "por             %%mm4, %%mm3 \n\t"                     //mm3|=mm4

                        "paddw   %%mm3, %%mm1        \n\t"              //mm1+=mm2
                        "movq    %%mm1, %%mm6        \n\t"              //mm6=mm1

                        //2
                        "movq    4(%%esi), %%mm1     \n\t"      //mm1=*pnLine +1 pixel
                        "movq    12(%%esi), %%mm2    \n\t" //mm2=*pnLine +3 pixel

                        "pand    %%mm7, %%mm1        \n\t"              //mm1=mm1&masklum
                        "pand    %%mm5, %%mm2        \n\t"              //mm2=mm2&masklum

                        "movq    %%mm1, %%mm3        \n\t"              //mm3=mm1
                        "movq    %%mm2, %%mm4        \n\t"              //mm4=mm2

                        "psrld   $16, %%mm3          \n\t"              //mm3=mm3>>16   (d)
                        "pslld   $16, %%mm4          \n\t"              //mm4=mm4<<16   (d)

                        "pand    %3, %%mm1 \n\t"      //mm1=mm1&mask0000ffff
                        "pand    %4, %%mm2 \n\t"      //mm2=mm2&maskffff0000

                        "por             %%mm2, %%mm1 \n\t"                     //mm1|=mm2
                        "por             %%mm4, %%mm3 \n\t"                     //mm3|=mm4

                        "paddw   %%mm3, %%mm1        \n\t"              //mm1+=mm2
                        //
                        "psubw   %%mm6, %%mm1        \n\t"              //mm1=mm1-mm2 (w)

                        "addl            $16, %%esi  \n\t"
                        "pmaddwd %%mm1, %%mm1        \n\t"              //mm1=mm1*mm1 (w) + (dw)

                        "decl %%ecx                  \n\t"
                        "paddd   %%mm1, %%mm0        \n\t"              //mm0 += mm1 (dw)

//                        "jnz CIPL.AutrePixel              \n\t"
                         "jnz 0b \n\t"

                        //add mm0
                        "movq            %%mm0, %%mm1 \n\t"
                        "punpckhdq       %%mm1, %%mm1 \n\t"
                        "paddd           %%mm1, %%mm0 \n\t"

                        "movd           %%mm0,%2       \n\t"

                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                     "m" (pnLine),"m" (nLoopPx), "m" (nSumDiff), "m" (mask0000ffff),  "m" (maskffff0000), "m" (mask00ff00ff)
                   : "%ecx", "%esi", "memory" // CHECKASM: clobber list
                );
                m_pnValueDiffPixelLine[py]=nSumDiff;
        }

        __asm__ __volatile__ ("emms");                              // output MMX mode


        //calucl corrective factor
        int nCorrFactor=0;
        for (i=0;i<m_nTestNormalMaxi;i++)
        {
                nCorrFactor=((CURRENTY/2)-abs(m_pnPositionLineNormal[i]-(CURRENTY/2)))*2/(CURRENTY/2)+1;        //2+1=3
        }       m_pnValueDiffPixelLine[i]=m_pnValueDiffPixelLine[i]*nCorrFactor;

        //sort by max
        for (i=0;i<m_nTestNormalMaxi;i++)
        {
                nBestScore=-1;
                position=0;
                for (j=0;j<m_nTestNormalMaxi;j++)
                {
                        if (m_pnValueDiffPixelLine[j]>nBestScore)
                        {
                                nBestScore=m_pnValueDiffPixelLine[j];
                                position=j;     //score max
                        }

                }
                m_psInterPixelNormal[i]=(short)position;        
                m_pnValueDiffPixelLine[position]=-1;

        }   
        

        //interlive
        for (i=0;i<(m_nTestNormalMaxi+1)/2;i++)
        {
                m_psInterliveSort[i*2]=m_psInterPixelNormal[i];
                m_psInterliveSort[i*2+1]=m_psInterPixelNormal[i+(m_nTestNormalMaxi+1)/2];
        }
        for (i=0;i<m_nTestNormalMaxi;i++)
                m_psInterPixelNormal[i]=m_psInterliveSort[i];

}


//save two trames for debug
bool CLaVache::SaveImage(short ** ppsBuffer1,short ** ppsBuffer2)
{
        FILE *sFile = fopen("CaptureMeuhMeuh.img", "wb");
        char phrase[]="Capture MeuhMeuhTV | ";
        char indent[40];

        if(sFile == NULL)
         return false;

        //identifiant
        ZeroMemory(indent,sizeof(indent));
        CopyMemory(indent,phrase,sizeof(phrase));

        fwrite(indent, sizeof(indent),1,sFile);

        //dimension x
        fwrite(&m_nSizeBufferX, sizeof(m_nSizeBufferX),1,sFile);

        //dimension y
        int nSizeY=m_nSizeBufferY*2;
        fwrite(&nSizeY, sizeof(nSizeY),1,sFile);

        //donnes image
        for (int py=0;py<m_nSizeBufferY;py++)
        {
                fwrite((ppsBuffer1[py]),m_nSizeBufferX*2,1,sFile);
                fwrite((ppsBuffer2[py]),m_nSizeBufferX*2,1,sFile);
        }

        fclose(sFile);

        return true;
}

void CLaVache::SetLoadImageMode(bool bUsedLoadImage)
{
        m_bUsedLoadImage=bUsedLoadImage;
}


//load two trames for debug
bool CLaVache::LoadImage()
{
        int tcx,tcy;
        int i;
        char phrase[]="Capture MeuhMeuhTV | ";
        char indent[40];
        char indent2[40];


        FILE *sFile = fopen("CaptureMeuhMeuh.img", "rb");

        if(sFile == NULL)
                return false;

        //identifiant
        ZeroMemory(indent,sizeof(indent));
        CopyMemory(indent,phrase,sizeof(phrase));

        fread(indent2, sizeof(indent2),1,sFile);
        for (i=0;i<40;i++)
        {
                if (indent[i]!=indent2[i])
                {
                        fclose(sFile);
                        return( false); 
                }
        }

        //dimension x
        fread(&tcx, sizeof(tcx),1,sFile);

        //dimension y
        fread(&tcy, sizeof(tcy),1,sFile);

        if ((tcx!=m_nSizeBufferX)|(tcy!=m_nSizeBufferY*2))
                return(false);

        //AllouRAM
        if (m_pnTamponChargeCapture)
                delete[] m_pnTamponChargeCapture;
        m_pnTamponChargeCapture=NULL;

        m_pnTamponChargeCapture=new int[tcx*tcy/2];
        if(!m_pnTamponChargeCapture)
                return false;

        //donnes image
        fread(m_pnTamponChargeCapture, tcx*tcy*2,1,sFile);

        fclose(sFile);

        return true;
}


void CLaVache::DirectBufferCopy(short ** ppsInputBuffer,short ** ppsOutputBuffer,BOOL bIsOdd)
{
        for (int py=0;py<CURRENTY;py++)
                ppsOutputBuffer[py]=ppsInputBuffer[py];

}


bool CLaVache::LoadFilePossiblePrevious()
{

        FILE *sFile = fopen(File_PossiblePrevious, "rb");

        if(sFile == NULL)
        {
                OutputDebugString("Impossible d'ouvrir le fichier accelerepos.fpc");
           return false;
        }

        if (!CheckFileID(sFile))
        {
                fclose(sFile);
                return false;
        }

        fread(m_pbPossiblePrevious, sizeof(*m_pbPossiblePrevious),CURRENTY*CURRENTY,sFile);

        fclose(sFile);


   return true;
}


bool CLaVache::LoadFileLinePrevious()
{
   int nZlr;
   FILE *sFile = fopen(File_LinePrevious, "rb");
   int nLineIndex;
   int nFullSize;
   short * adresse_clef;

   if(sFile == NULL)
      return false;

        if (!CheckFileID(sFile))
        {
                fclose(sFile);
                return false;
        }

   //line count
   fread(&m_nTestPreviousMaxi, sizeof(m_nTestPreviousMaxi),1,sFile);

   fread(&nFullSize, sizeof(nFullSize),1,sFile);

   m_psCountKeyLinePrevious=new short[nFullSize];
        if(m_psCountKeyLinePrevious == NULL)
        {
      OutputDebugString("IDS_ERR_MEMORY");
      return false;
        }

        m_pnPositionLinePrevious=new int[m_nTestPreviousMaxi];
        if(m_pnPositionLinePrevious == NULL)
        {
      OutputDebugString("IDS_ERR_MEMORY");
      return false;
        }


        m_ppsAdressKeyLinePrevious=new short* [m_nTestPreviousMaxi*CURRENTY*CURRENTY+1];
        if(m_ppsAdressKeyLinePrevious == NULL)
        {
      OutputDebugString("IDS_ERR_MEMORY");
      return false;
        }


        m_pcKeyLinePrevious=new unsigned char[m_nTestPreviousMaxi*CURRENTY*CURRENTY];
        if(m_pcKeyLinePrevious == NULL)
        {
      OutputDebugString("IDS_ERR_MEMORY");
      return false;
        }


        fread(m_psCountKeyLinePrevious, sizeof(*m_psCountKeyLinePrevious),nFullSize,sFile);

   
   adresse_clef=m_psCountKeyLinePrevious;
   for (nLineIndex=0;nLineIndex<m_nTestPreviousMaxi;nLineIndex++)
   {
           //record line number
           fread(m_pnPositionLinePrevious+nLineIndex, sizeof(*m_pnPositionLinePrevious),1,sFile);

           //record key count ,line
           fread(m_pcKeyLinePrevious+nLineIndex*CURRENTY*CURRENTY, sizeof(*m_pcKeyLinePrevious),CURRENTY*CURRENTY,sFile);

           for(nZlr=0; nZlr<CURRENTY*CURRENTY; nZlr++)
           {
                   //calcul pointers
                        m_ppsAdressKeyLinePrevious[nLineIndex*CURRENTY*CURRENTY+nZlr]=adresse_clef;
                        adresse_clef+=*(m_pcKeyLinePrevious+nLineIndex*CURRENTY*CURRENTY+nZlr);
           }

   }

   fclose(sFile);
   return true;
}


bool CLaVache::FindKeyPrevious(int nLineIndex)
{

        int nKeyTested,nIndexKeyTested;
        int nBestScore;
        int nPropaGoodKey;
        int i;
        int ProbaOrdreLigneDecodeM0;
        int ProbaOrdreLigneDecodeM4;
        int nKeyKeep=0;
        int nCurrentColumnTest;

        int nOldBadKey=m_nKeyFound;

        if ((m_pnPositionLinePrevious[nLineIndex]<m_nHeightBlackBorder)
                ||(m_pnPositionLinePrevious[nLineIndex]>CURRENTY-m_nHeightBlackBorder))
        {
                return(false);  //over line
        }
        if ((m_pnPositionLinePrevious[nLineIndex]+ECARTLIGNESUPERPOSITION<m_nHeightBlackBorder)
                ||(m_pnPositionLinePrevious[nLineIndex]+ECARTLIGNESUPERPOSITION>CURRENTY-m_nHeightBlackBorder))
        {
                return(false);  //over line
        }

        //search lines near in the previous farme
        if (!FindLinePrevious(m_pnPositionLinePrevious[nLineIndex],-1,ProbaOrdreLigneDecodeM0))
                return(false);

        if (!FindLinePrevious(m_pnPositionLinePrevious[nLineIndex]+ECARTLIGNESUPERPOSITION,ProbaOrdreLigneDecodeM0,ProbaOrdreLigneDecodeM4))
                return(false);


        if (m_pcKeyLinePrevious[nLineIndex*CURRENTY*CURRENTY+ProbaOrdreLigneDecodeM0*CURRENTY+ProbaOrdreLigneDecodeM4]==0)
                return(false);


        if (m_pnDiffColumnPrevious[0]==-1)
        {       //m_pnDiffColumnNormal wasn't yet calculed

                for (int py=0;py<CURRENTY;py++)
                        m_psSortLineCurrent[py]=py;

                //calcul nPropaGoodKey
                nPropaGoodKey=0;
                for (i=0;i<NCOLUMNTESTED;i++)   
                {       //1 passe
                        nPropaGoodKey+=CalculDiffColumnPrevious(i,0x7fffffff);
                        m_pnDiffColumnPrevious[i]=nPropaGoodKey;
                }
        }


        nCurrentColumnTest=m_nTestDone/2+3;
        if (nCurrentColumnTest>NCOLUMNTESTED)
                nCurrentColumnTest=NCOLUMNTESTED;


        //DifferenceColonnePrecedent
        nBestScore=0x0FFFFFFF;
        for (nIndexKeyTested=0;nIndexKeyTested<m_pcKeyLinePrevious[nLineIndex*CURRENTY*CURRENTY+ProbaOrdreLigneDecodeM0*CURRENTY+ProbaOrdreLigneDecodeM4];nIndexKeyTested++)
        {
                nKeyTested=*(m_ppsAdressKeyLinePrevious[nLineIndex*CURRENTY*CURRENTY+ProbaOrdreLigneDecodeM0*CURRENTY+ProbaOrdreLigneDecodeM4]+nIndexKeyTested);
                CreateLineOrder(nKeyTested,m_psSortLineCurrent);

                nPropaGoodKey=0;
                for (i=0;i<nCurrentColumnTest;i++)
                {       
                        nPropaGoodKey+=CalculDiffColumnPrevious(i,nBestScore+10);       

                        if (nPropaGoodKey>nBestScore)
                                break;  //over -> out

                }

                                        
                if (nPropaGoodKey<nBestScore)
                {
                        nBestScore=nPropaGoodKey;
                        nKeyKeep=nKeyTested;
                }

                if (nIndexKeyTested>2)
                {
                        m_nRateGoodProba=(m_pnDiffColumnPrevious[nCurrentColumnTest-1]-nBestScore)*100/(nBestScore+1);

                        if (m_nRateGoodProba>170+m_nProbaPreviousDynam) 
                                break;  //very good -> out
                }

        }

        __asm__ __volatile__   ("emms");                           // output MMX mode


        // Validation ?
        m_nTypeTryPrevious=1;

        if (nIndexKeyTested>0)
        {
                m_nRateGoodProba=(m_pnDiffColumnPrevious[nCurrentColumnTest-1]-nBestScore)*100/(nBestScore+1);
                if (m_nRateGoodProba<0)
                        m_nRateGoodProba=0;

                m_nKeyFound=(short)nKeyKeep;

                if (m_nRateGoodProba>40+m_nProbaPreviousDynam-m_pcKeyAlreadyFound[nKeyKeep]*50)
                {
                        CreateLineOrder(nKeyKeep,m_psSortLineCurrent);

                        m_bKeyFound=true;
                        m_nTestDone++;
                
                        m_nTypeTryPrevious=2;   // => fixe frame                                        
                        m_nLineIndexSelectPrevious=nLineIndex;
        
                        //limit
                        m_nProbaPreviousLimitSup=(m_nProbaPreviousLimitSup*COEFFILTERLIMIT+m_nRateGoodProba*COEFFILTERLIMIT)/(COEFFILTERLIMIT+1);
                        if (m_nProbaPreviousLimitSup>TresholdPreviousLimitSup)
                                m_nProbaPreviousLimitSup=TresholdPreviousLimitSup;

                        return(true);
                }

        }


        //limit
        if ((m_nTestDone>0)&&(nOldBadKey!=m_nKeyFound)&&(m_pcKeyAlreadyFound[nKeyKeep]==0))
        {
                m_nProbaPreviousLimitInf=(m_nProbaPreviousLimitInf*COEFFILTERLIMIT+m_nRateGoodProba*COEFFILTERLIMIT)/(COEFFILTERLIMIT+1);
                if (m_nProbaPreviousLimitInf>TresholdPreviousLimitInf)
                        m_nProbaPreviousLimitInf=TresholdPreviousLimitInf;
        }

        //incrise m_pcKeyAlreadyFound[nKeyKeep]
        m_pcKeyAlreadyFound[nKeyKeep]++;

        m_nTestDone++;
        return(false);

        //m_nTypeTryPrevious=2 -> fixe frame
        //m_nTypeTryPrevious=1 -> moving frame
        //m_nTypeTryPrevious=0 -> we don't know


}


bool CLaVache::FindLinePrevious(int nLine,int nLineNotUsed,int & nLineActualFound)
{       //search lines near in the previous frame

        int py;
        int nBestScore,nBestLine;
        int * pnLinePrevious,* pnLineActualTest;
        int nSumDiff;


        pnLinePrevious=m_pnLineOldSort288[m_bIsOdd][nLine];  

        nBestLine=0;
        nBestScore=0x0FFFFFFF;
        for (py=0;py<CURRENTY;py++)
        {
                if ((m_pbPossiblePrevious[nLine*CURRENTY+py])&&(py!=nLineNotUsed))
                {
                        nSumDiff=0;
                        pnLineActualTest=m_pnLineCurent288[py];

                        CalculDiffLinePrevious(pnLineActualTest,pnLinePrevious,nSumDiff,nBestScore);

                        if (nSumDiff<nBestScore)
                        {
                                nBestScore=nSumDiff;
                                nBestLine=py;

                        }
                }
        }

        __asm__ __volatile__("emms");                              // output MMX mode

        nLineActualFound=nBestLine;
        
        if (nBestScore==0x0FFFFFFF)
                return( false);         //no line selected

        return(true);

}




void CLaVache::CalculDiffLinePrevious(int * pnLine1Current,int * pnLine2Near,int &nSumDiff,int nSumDiff_max)
{
        const int balayage_diff_pre=4;
        const int saut=balayage_diff_pre*8;
        // const __int64 maskdw  = 0x00000000FFFFFFFFLL;
        // const __int64 maskw12 = 0x0000FFFF0000FFFFLL;
        const __int64 masklum = 0x00FF00FF00FF00FFLL;

        int * pnLine1,* pnLine2;
        int nLoopPx;
        int phase;
        int nSumDiff_intermediaire=0;

        nSumDiff=0;

        nLoopPx=m_nSizeBufferX*sizeof(short)/sizeof(__int64)/balayage_diff_pre;

        for (phase=0;phase<balayage_diff_pre;phase+=1)  //2 dword
        {
                pnLine1=pnLine1Current+phase*2; // 2 dword
                pnLine2=pnLine2Near+phase*2;    // 2 dword

                  __asm__ __volatile__ (
                        "movl            %0, %%edi \n\t"   //courante
                        "pxor    %%mm0, %%mm0        \n\t" // mm0=0

                        "movl            %1, %%esi \n\t"   //avant
                        "movq    %5, %%mm7      \n\t"      //mm7=masklum
                        "movq    %%mm7, %%mm6        \n\t"      //mm6=masklum

                        "movl           %2, %%ecx       \n\t"
                        "movl           %3, %%eax       \n\t"
                      //  "jmp             CDLP.AutrePixel_diff_pre\n\t"
                        "jmp 0f \n\t"

                        ".align 16                   \n\t"
                       // "CDLP.AutrePixel_diff_pre:        \n\t"
                        "0: \n\t"
                        "movq    (%%edi), %%mm1      \n\t"      //mm1=*pnLine1
                        "movq    (%%esi), %%mm2      \n\t"      //mm2=*pnLine2

                        "pand    %%mm6, %%mm1        \n\t"              //mm1'=mm1&masklum
                        "pand    %%mm7, %%mm2        \n\t"              //mm2'=mm2&masklum

                        "movq    %%mm1, %%mm3        \n\t"              //mm3=mm1
                        "movq    %%mm2, %%mm4        \n\t"              //mm4=mm2

                        "psubusw %%mm2, %%mm1        \n\t"              //mm1=mm1-mm2 -> =0 si ngatif
                        "addl            %%eax, %%esi \n\t"                     //2 dword

                        "psubusw %%mm3, %%mm4        \n\t"              //mm4=mm2-mm1 -> =0 si ngatif
                        "addl            %%eax, %%edi \n\t"                     //2 dword

                        "por             %%mm4, %%mm1 \n\t"                     //mm1=mm1+mm4 -> mm1=|mm1'-mm2'|
                        "decl            %%ecx       \n\t"

                        "paddw   %%mm1, %%mm0        \n\t"
//                        "jnz             CDLP.AutrePixel_diff_pre\n\t"
                        "jnz 0b \n\t"
                        //additionne mm0
                        "movq            %%mm0, %%mm1 \n\t"             //mm1=mm0= A B C D (w)
                        "pxor            %%mm2, %%mm2 \n\t"             //mm2    = 0 0 0 0 (w)

                        "punpcklwd       %%mm2, %%mm0 \n\t"             //mm0=     0 C 0 D (w)
                        "punpckhwd       %%mm2, %%mm1 \n\t"             //mm1=     0 A 0 B (w)

                        "paddd           %%mm1, %%mm0 \n\t"             //mm0= (A+C)  (B+D) (dw)

                        "movq            %%mm0, %%mm1 \n\t"             //mm1= (A+C)  (B+D) (dw)
                        "punpckhdq       %%mm1, %%mm1 \n\t"             //mm1= (A+C)  (A+C) (dw)
                        "paddd           %%mm1, %%mm0 \n\t"             //mm0=   X    (A+B+C+D) (dw)

                        "movd           %%mm0,%4       \n\t"


                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                     "m"(pnLine1), "m"(pnLine2),  "m"(nLoopPx),  "m"(saut), "m"(nSumDiff_intermediaire),"m" (masklum)
                   : "%eax", "%ecx", "%edi", "%esi", "memory" // CHECKASM: clobber list
                );


                nSumDiff+=nSumDiff_intermediaire;

                if (nSumDiff>nSumDiff_max)
                        break;  //over  -> out

        }

}



int CLaVache::CalculDiffColumnPrevious(int nIndexCulumn,int nMaxValue)
{ 

        const __int64 masklum = 0x00FF00FF00FF00FFLL;
        int nSumDiffVabs = 0;
        int nThresholdMax=nMaxValue/2;  // divid by 2 because compare do in the half of nSumDiff (mm7)

        int *pValueColumnActual=&(m_pnColumnValue[nIndexCulumn][0]);
        short * psSortActual=&m_psSortLineCurrent[284];

        int *pValueColumnOldSort=&(m_pnColumnOldSortValue[m_bIsOdd][nIndexCulumn][284]);

          __asm__ __volatile__ (
                "movl                    %0, %%esi \n\t"
                "movq            %5, %%mm6 \n\t"                   //mm6=masklum

                "movl                    %1, %%edi \n\t"
                "xorl                    %%eax, %%eax \n\t"                             //exa=0

                "movl                    %2, %%edx \n\t"               //edx= &(m_pnColumnOldSortValue[m_bIsOdd][284])
                "pxor            %%mm7, %%mm7 \n\t"                     // mm7 = nSumDiffVabs = 0

                "movl                    $284, %%ecx \n\t"                      //py=284        (multiple de 4)
                "movq            %%mm6, %%mm5 \n\t"                     //mm5=masklum

                // "jmp                     CDCP.Boucle1 \n\t"
		"jmp 0f \n\t"
                ".align 16                   \n\t"
// "CDCP.Boucle1:                    \n\t"
                "0: \n\t"

                //----
                "movw                    (%%edi), %%ax \n\t"
                "movd            (%%edx), %%mm1 \n\t"                   //mm1 = 00000000 00000000 00000000 00000000  VVVVVVVV y2y2y2y2 UUUUUUUU y1y1y1y1
                "movd            (%%esi,%%eax,4), %%mm0 \n\t" //mm0 = 00000000 00000000 00000000 00000000  VVVVVVVV Y2Y2Y2Y2 UUUUUUUU Y1Y1Y1Y1

                "movw                    -2(%%edi), %%ax \n\t"
                "punpckldq       -4(%%edx), %%mm1 \n\t"                 //mm1 = VVVVVVVV y4y4y4y4 UUUUUUUU y3y3y3y3  VVVVVVVV y2y2y2y2 UUUUUUUU y1y1y1y1
                "punpckldq       (%%esi,%%eax,4), %%mm0 \n\t" //mm0 = VVVVVVVV Y4Y4Y4Y4 UUUUUUUU Y3Y3Y3Y3  VVVVVVVV Y2Y2Y2Y2 VVVVVVVV Y1Y1Y1Y1

                "pand            %%mm5, %%mm1 \n\t"                     //mm1 = 00000000 y4y4y4y4 00000000 y3y3y3y3  00000000 y2y2y2y2 00000000 y2y2y2y2
                "pand            %%mm6, %%mm0 \n\t"                     //mm0 = 00000000 Y4Y4Y4Y4 00000000 Y3Y3Y3Y3  00000000 Y2Y2Y2Y2 00000000 Y1Y1Y1Y1
                "psubw           %%mm1, %%mm0 \n\t"                     //mm0 =            Y4-y4             Y3-y3              Y2-y2              Y1-y1
                "pmaddwd         %%mm0, %%mm0 \n\t"                     //mm0 =     (Y4-y4)^2 + (Y3-y3)^2                (Y2-y2)^2 + (Y1-y1)^2
                "paddd           %%mm0, %%mm7 \n\t"                     //mm7 += mm0

                //----
                "movw                    2(%%edi), %%ax \n\t"
                "movd            4(%%edx), %%mm1 \n\t"                  //mm1 = 00000000 00000000 00000000 00000000  VVVVVVVV y2y2y2y2 UUUUUUUU y1y1y1y1
                "movd            (%%esi,%%eax,4), %%mm0 \n\t" //mm0 = 00000000 00000000 00000000 00000000  VVVVVVVV Y2Y2Y2Y2 UUUUUUUU Y1Y1Y1Y1

                "movw                    4(%%edi), %%ax \n\t"
                "punpckldq       8(%%edx), %%mm1 \n\t"                  //mm1 = VVVVVVVV y4y4y4y4 UUUUUUUU y3y3y3y3  VVVVVVVV y2y2y2y2 UUUUUUUU y1y1y1y1
                "punpckldq       (%%esi,%%eax,4), %%mm0 \n\t" //mm0 = VVVVVVVV Y4Y4Y4Y4 UUUUUUUU Y3Y3Y3Y3  VVVVVVVV Y2Y2Y2Y2 VVVVVVVV Y1Y1Y1Y1

                "pand            %%mm5, %%mm1 \n\t"                     //mm1 = 00000000 y4y4y4y4 00000000 y3y3y3y3  00000000 y2y2y2y2 00000000 y2y2y2y2
                "pand            %%mm6, %%mm0 \n\t"                     //mm0 = 00000000 Y4Y4Y4Y4 00000000 Y3Y3Y3Y3  00000000 Y2Y2Y2Y2 00000000 Y1Y1Y1Y1
                "psubw           %%mm1, %%mm0 \n\t"                     //mm0 =            Y4-y4             Y3-y3              Y2-y2              Y1-y1

                "subl                    $16, %%edx \n\t"                       //edx-=16
                "pmaddwd         %%mm0, %%mm0 \n\t"                     //mm0 =     (Y4-y4)^2 + (Y3-y3)^2                (Y2-y2)^2 + (Y1-y1)^2

                "subl                    $8, %%edi \n\t"                        //edi-=8
                "paddd           %%mm0, %%mm7 \n\t"                     //mm7 += mm0

                //----
                "movd            %%mm7, %%ebx \n\t"
                "subl             %3,%%ebx \n\t"
//                "jg                      CDCP.Boucle2 \n\t"                          //dpacement Threshold -> fin boucle
                "jg 1f \n\t"

                "subl                    $4, %%ecx \n\t"                        //py-=4
		//"jnz                     CDCP.Boucle1 \n\t"
                "jnz 0b \n\t"

// "CDCP.Boucle2:                    \n\t"
                "1: \n\t"
		"movq            %%mm7, %%mm0 \n\t"
                "punpckhdq       %%mm0, %%mm0 \n\t"
                "paddd           %%mm7, %%mm0 \n\t"
                "movd            %%mm0, %4 \n\t"


           : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

           : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
             "m" (pValueColumnActual), "m" (psSortActual),"m" (pValueColumnOldSort),"m" (nThresholdMax), "m" (nSumDiffVabs),"m" (masklum)
           : "%eax", "%ebx", "%ecx", "%edx", "%edi", "%esi", "memory" // CHECKASM: clobber list
        );

        return(nSumDiffVabs);
}

void CLaVache::CalculInterPixelLinePrevious()
{
        int i,j,py;
        int * pnLine;
        int nBestScore,position;
        int nSumDiff=0;

        
        int nLoopPx=m_nSizeBufferX*sizeof(short)/sizeof(__int64)/2-2;
        const __int64 mask0000ffff = 0x0000ffff0000ffffLL;
        const __int64 maskffff0000 = 0xffff0000ffff0000LL;
        const __int64 mask00ff00ff = 0x00ff00ff00ff00ffLL;


        for (py=0;py<m_nTestPreviousMaxi;py++)
        {
                pnLine=m_pnLineOldSort288[m_bIsOdd][m_pnPositionLinePrevious[py]];

                  __asm__ __volatile__ (
                        "pxor    %%mm0, %%mm0        \n\t"      // mm0=0

                        "movl            %0, %%esi \n\t"
                        "movq    %5, %%mm7 \n\t" //mm7=masklum
                        "movq    %%mm7, %%mm5        \n\t"      //mm5=masklum

                        "movl            %1,%%ecx       \n\t"
//                        "jmp             CIPLP.AutrePixel  \n\t"
                        "jmp 0f \n\t"

                        ".align 16                   \n\t"
//                "CIPLP.AutrePixel:                 \n\t"
                        "0: \n\t"

                        //1
                        "movq    (%%esi), %%mm1      \n\t"      //mm1=*pnLine
                        "movq    8(%%esi), %%mm2     \n\t"      //mm2=*pnLine +2 pixel

                        "pand    %%mm7, %%mm1        \n\t"              //mm1=mm1&masklum
                        "pand    %%mm5, %%mm2        \n\t"              //mm2=mm2&masklum

                        "movq    %%mm1, %%mm3        \n\t"              //mm3=mm1
                        "movq    %%mm2, %%mm4        \n\t"              //mm4=mm2

                        "psrld   $16, %%mm3          \n\t"              //mm3=mm3>>16   (d)
                        "pslld   $16, %%mm4          \n\t"              //mm4=mm4<<16   (d)

                        "pand    %3, %%mm1 \n\t"      //mm1=mm1&mask0000ffff
                        "pand    %4, %%mm2 \n\t"      //mm2=mm2&maskffff0000

                        "por             %%mm2, %%mm1 \n\t"                     //mm1|=mm2
                        "por             %%mm4, %%mm3 \n\t"                     //mm3|=mm4

                        "paddw   %%mm3, %%mm1        \n\t"              //mm1+=mm2
                        "movq    %%mm1, %%mm6        \n\t"              //mm6=mm1

                        //2
                        "movq    4(%%esi), %%mm1     \n\t"      //mm1=*pnLine +1 pixel
                        "movq    12(%%esi), %%mm2    \n\t" //mm2=*pnLine +3 pixel

                        "pand    %%mm7, %%mm1        \n\t"              //mm1=mm1&masklum
                        "pand    %%mm5, %%mm2        \n\t"              //mm2=mm2&masklum

                        "movq    %%mm1, %%mm3        \n\t"              //mm3=mm1
                        "movq    %%mm2, %%mm4        \n\t"              //mm4=mm2

                        "psrld   $16, %%mm3          \n\t"              //mm3=mm3>>16   (d)
                        "pslld   $16, %%mm4          \n\t"              //mm4=mm4<<16   (d)

                        "pand    %3, %%mm1 \n\t"      //mm1=mm1&mask0000ffff
                        "pand    %4, %%mm2 \n\t"      //mm2=mm2&maskffff0000

                        "por             %%mm2, %%mm1 \n\t"                     //mm1|=mm2
                        "por             %%mm4, %%mm3 \n\t"                     //mm3|=mm4

                        "paddw   %%mm3, %%mm1        \n\t"              //mm1+=mm2
                        //
                        "psubw   %%mm6, %%mm1        \n\t"              //mm1=mm1-mm2 (w)

                        "addl            $16, %%esi  \n\t"
                        "pmaddwd %%mm1, %%mm1        \n\t"              //mm1=mm1*mm1 (w) + (dw)

                        "decl %%ecx                  \n\t"
                        "paddd   %%mm1, %%mm0        \n\t"              //mm0 += mm1 (dw)

//                        "jnz CIPLP.AutrePixel              \n\t"
                        "jnz 0b \n\t"

                        //additionne mm0
                        "movq            %%mm0, %%mm1 \n\t"
                        "punpckhdq       %%mm1, %%mm1 \n\t"
                        "paddd           %%mm1, %%mm0 \n\t"

                        "movd           %%mm0,%2       \n\t"

                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                    "m" (pnLine),  "m" (nLoopPx), "m" (nSumDiff), "m" (mask0000ffff), "m" (maskffff0000), "m" (mask00ff00ff)
                   : "%ecx", "%esi","memory"  // CHECKASM: clobber list
                );
                m_pnValueDiffPixelLine[py]=nSumDiff;
        }

        __asm__ __volatile__ ("emms");                              // output MMX mode


        //calucl corrective factor
        int nCorrFactor=0;
        for (i=0;i<m_nTestPreviousMaxi;i++)
        {
                nCorrFactor=((CURRENTY/2)-abs(m_pnPositionLinePrevious[i]-(CURRENTY/2)))*2/(CURRENTY/2)+1;      //2+1=3
        }       m_pnValueDiffPixelLine[i]=m_pnValueDiffPixelLine[i]*nCorrFactor;


        //sort max
        int position_a_remplacer=0;
        for (i=0;i<m_nTestPreviousMaxi;i++)
        {
                nBestScore=-1;
                position=0;
                for (j=0;j<m_nTestPreviousMaxi;j++)
                {
                        if (m_pnValueDiffPixelLine[j]>nBestScore)
                        {
                                nBestScore=m_pnValueDiffPixelLine[j];
                                position=j;     //score max
                        }

                }
                m_psInterPixelPrevious[i]=(short)position;      
                m_pnValueDiffPixelLine[position]=-1;
                if (m_psInterPixelPrevious[i]==m_nLineIndexSelectPrevious)
                        position_a_remplacer=i;
        }   



        //set p_Trame_Proche->m_nLineIndexSelectPrevious in fisrt
        int index_premier=m_nLineIndexSelectPrevious;
        if (index_premier!=-1)
        {
                m_psInterPixelPrevious[position_a_remplacer]=m_psInterPixelPrevious[0];
                m_psInterPixelPrevious[0]=index_premier;
        }


        //Interlive
        for (i=0;i<(m_nTestPreviousMaxi+1)/2;i++)
        {
                m_psInterliveSort[i*2]=m_psInterPixelPrevious[i];
                m_psInterliveSort[i*2+1]=m_psInterPixelPrevious[i+(m_nTestPreviousMaxi+1)/2];
        }
        for (i=0;i<m_nTestPreviousMaxi;i++)
                m_psInterPixelPrevious[i]=m_psInterliveSort[i];


}


//debug
void CLaVache::ShowColumn()             
{
        int py,nColumn;

        for (py=0;py<CURRENTY;py++)
        {
                for (nColumn=0;nColumn<NCOLUMNTESTED;nColumn++)
                        *(m_pnLineCurent288[py]+m_pnColumnSelect[nColumn]+1)=0xFFFFFFFF;
        }

/*
        *(m_pnLineCurent288[m_pnPositionLinePrevious[m_nLineIndexSelectPrevious]]+0)=0xFFFFFFFF;
        *(m_pnLineCurent288[m_pnPositionLinePrevious[m_nLineIndexSelectPrevious]]+1)=0xFFFFFFFF;
        *(m_pnLineCurent288[m_pnPositionLinePrevious[m_nLineIndexSelectPrevious]]+2)=0xFFFFFFFF;
        *(m_pnLineCurent288[m_pnPositionLinePrevious[m_nLineIndexSelectPrevious]]+3)=0xFFFFFFFF;
        *(m_pnLineCurent288[m_pnPositionLinePrevious[m_nLineIndexSelectPrevious]]+4)=0xFFFFFFFF;
        *(m_pnLineCurent288[m_pnPositionLinePrevious[m_nLineIndexSelectPrevious]]+5)=0xFFFFFFFF;
*/
}

bool CLaVache::IsFrameFixe()
{
        int nColumnIndex1,nColumnIndex2;
        int nColumnID=0;

        for (nColumnIndex1=0;nColumnIndex1<NCOLUMNTESTED;nColumnIndex1++)
        {
                for (nColumnIndex2=0;nColumnIndex2<NCOLUMNTESTED;nColumnIndex2++)
                {
                        if (m_pnColumnOldSelect[m_bIsOdd][nColumnIndex1]==m_pnColumnSelect[nColumnIndex2])
                                nColumnID++;
                }
        }

        if (nColumnID>=5)
                return true;
        else 
                return false;
}


void CLaVache::RecordKeyRepeat(bool bKeyFound,int nKeyFound)
{
        if (bKeyFound)
        {
                if ((!m_bKeyRepeatLock)&&(m_psKeyRepeat[m_nPositionKeyRepeat]==nKeyFound))
                        m_nGoodKeySynchrone++;
                else
                        m_nGoodKeySynchrone=0;

                if (m_nGoodKeySynchrone==FREQREPEATKEY)
                {
                        m_bKeyRepeatLock=true;  //lock !
                        m_nPositionKeyRepeatFound=m_nPositionKeyRepeat+1;
                        if (m_nPositionKeyRepeatFound>=FREQREPEATKEY)
                                m_nPositionKeyRepeatFound=0;
                        m_bKeyRepeatRead=true; //read key
                        m_nBadKeySynchrone=0;
                        m_nGoodKeySynchrone=0;
                }


                if ((m_bKeyRepeatLock)&&(m_bKeyRepeatRead)&&(m_psKeyRepeat[m_nPositionKeyRepeat]!=nKeyFound))
                        m_nBadKeySynchrone++;

                m_psKeyRepeat[m_nPositionKeyRepeat]=nKeyFound;
        }
        else
        {
                m_psKeyRepeat[m_nPositionKeyRepeat]=-1;
        }


        m_nPositionKeyRepeat++;
        if (m_nPositionKeyRepeat>=FREQREPEATKEY)
                m_nPositionKeyRepeat=0;


        if (m_nBadKeySynchrone>3)
        {
                //m_nBadKeySynchrone=0;
                //m_bKeyRepeatLock=false;               //bad key -> UnLock !
                LossKeyRepeat();                                //bad key -> UnLock !
        }

        if ((m_bKeyRepeatLock)&&(m_nPositionKeyRepeatFound==m_nPositionKeyRepeat))
        {
                if ((m_bKeyRepeatRead)&&(m_nBadKeySynchroneOld==m_nBadKeySynchrone)&&(m_nBadKeySynchrone>0))
                        m_nBadKeySynchrone--;

                m_bKeyRepeatRead=!m_bKeyRepeatRead;             //toggle m_bKeyRepeatRead

                m_nBadKeySynchroneOld=m_nBadKeySynchrone;
        }
}


void CLaVache::LossKeyRepeat()
{
                m_nBadKeySynchrone=0;
                m_bKeyRepeatLock=false;         //unlock key repeat
}


bool CLaVache::UseKeyRepeat()
{
        int nDistance=abs(m_nPositionKeyRepeat-m_nPositionKeyRepeatFound);
        if (nDistance>FREQREPEATKEY/2)
                nDistance=FREQREPEATKEY-nDistance;

        if ((m_bKeyRepeatLock)
                &&(m_bKeyRepeatRead)
                &&(m_psKeyRepeat[m_nPositionKeyRepeat]!=-1)
                &&(nDistance>1))
        {
                m_nKeyFound=m_psKeyRepeat[m_nPositionKeyRepeat];
                CreateLineOrder(m_nKeyFound,m_psSortLineCurrent);
                m_nTypeTryPrevious=1;
                m_bKeyFound=true;
        }

        return m_bKeyFound;
}


void CLaVache::SetParams(int nTestNormalUser,int nTestPreviousUser,bool bFilterSECAM,bool bFilterPurple, bool bFilterAverage)
{
        m_nTestNormalUser=nTestNormalUser;
        m_nTestPreviousUser=nTestPreviousUser;
        if ((nTestNormalUser<0)||(nTestNormalUser>m_nTestNormalMaxi))
                nTestNormalUser=0;
        if ((nTestPreviousUser<0)||(nTestPreviousUser>m_nTestPreviousMaxi))
                nTestPreviousUser=0;

        m_nLineIndexSelectPrevious=m_nTestPreviousMaxi/2;

        m_bFilterSECAM=bFilterSECAM;
        m_bFilterPurple=bFilterPurple;
        m_bFilterAverage=bFilterAverage;
}


bool CLaVache::DetectLineBlue(int * pnLine)
{
        const int ThresholdSaturation=145;      //150

        int ThresholdPixelMiss=(m_nSizeBufferX/2*20)/100;       // 20%
        int px;
        int nColorCurrent;
        int nSaturationMiss;


        nSaturationMiss=0;
        for (px=0;px<m_nSizeBufferX/2;px++)     
        {
                nColorCurrent=*(pnLine+px);
                if ((unsigned char)(nColorCurrent>>8)<ThresholdSaturation )
                {       // non bleu
                        nSaturationMiss++;
        
                        if (nSaturationMiss>ThresholdPixelMiss) 
                                break;
                }

        }

        if (nSaturationMiss<=ThresholdPixelMiss)
                return true;

        return false;
}


bool CLaVache::DetectLineRed(int * pnLine)
{
        const int ThresholdSaturation=145;      //150

        int ThresholdPixelMiss=(m_nSizeBufferX/2*20)/100;       // 20%
        int px;
        int nColorCurrent;
        int nSaturationMiss;


        nSaturationMiss=0;
        for (px=0;px<m_nSizeBufferX/2;px++)     
        {
                nColorCurrent=*(pnLine+px);
                if ((unsigned char)(nColorCurrent>>24)<ThresholdSaturation )
                {       //non rouge 
                        nSaturationMiss++;
        
                        if (nSaturationMiss>ThresholdPixelMiss)
                                break;
                }

        }

        if (nSaturationMiss<=ThresholdPixelMiss)
                return true;

        return false;
}

void CLaVache::FilterFrame(bool bPhaseFrame) 
{
        if (m_bFilterPurple)
                FilterPurple();

        if (m_bFilterSECAM)
                FilterSecam(bPhaseFrame);

        if (m_bFilterAverage)
                FilterAverage();
}


void CLaVache::FilterAverage()
{
        int py;
        int * pnLineCurrent;
        int * pnLinePrevious;
        int nLoopPx;
        __int64 masklum = 0x00FF00FF00FF00FFLL;
        __int64 maskcoul = 0xFF00FF00FF00FF00LL;

        if (!m_bOldKeyFound[m_bIsOdd])
                return;
        
          __asm__  __volatile__ (
                        "movq    %0, %%mm4      \n\t"
                        "movq    %1, %%mm5     \n\t"
                        "movq    %%mm5, %%mm6        \n\t"
                        : : "m" (masklum), "m" (maskcoul) : "memory"

        );
        

        //nLoopPx=sCaptureVideo[nCaptureVideoSelect].TailleVideoLigneOctet/sizeof(__int64);     
        nLoopPx=m_nSizeBufferX*sizeof(short)/sizeof(__int64);

        for (py=0+m_nHeightBlackBorder;py<CURRENTY-m_nHeightBlackBorder;py++)
        {
                pnLineCurrent=m_pnLineSort288[py];
                pnLinePrevious=m_pnLineOldSort288[m_bIsOdd][py];
                
                  __asm__ __volatile__ (
                                //Pour YUV2
                        "movl            %0, %%esi \n\t"
                        "movl            %1, %%edi \n\t"

                        "movl            %2, %%ecx       \n\t"

//                        "jmp             FA.AutrePixel  \n\t"
                        "jmp 0f \n\t"
                        ".align 16                   \n\t"
// "FA.AutrePixel:                 \n\t"
                        "0: \n\t"
                        "movq    (%%esi), %%mm0      \n\t"              //mm0=ligne_precedante
                        "movq    (%%edi), %%mm1      \n\t"              //mm1=ligne_courante                            

                        //moyenne couleur
                        "psrlw   $1, %%mm0           \n\t"                      //mm0=mm0 >>1 (w)
                        "movq    %%mm1, %%mm2        \n\t"                      //mm2=ligne_courante

                        "psrlw   $1, %%mm1           \n\t"                      //mm1=mm1 >>1 (w)
                        "pand    %%mm4, %%mm2        \n\t"                      //mm2=mm2&masklum

                        "pand    %%mm5, %%mm0        \n\t"                      //mm0=mm0&maskcoul
                        "pand    %%mm6, %%mm1        \n\t"                      //mm1=mm1&maskcoul

                        "addl            $8, %%edi   \n\t"
                        "paddw   %%mm1, %%mm0        \n\t"

                        "addl            $8, %%esi   \n\t"
                        "por             %%mm2, %%mm0 \n\t"

                        "decl            %%ecx       \n\t"
                        //stockage
                        "movq    %%mm0, -8(%%edi)    \n\t"
//                        "jnz             FA.AutrePixel  \n\t"
                        "jnz 0b \n\t"

                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                    "m" (pnLinePrevious), "m" (pnLineCurrent), "m" (nLoopPx)
                   : "%ecx", "%edi", "%esi", "memory" // CHECKASM: clobber list
                );
                
        }

        
        __asm__    ("emms");          //ouput mmx mode

}


void CLaVache::FilterSecam(bool bPhaseFrame) 
{       //original idea : l'1Konnu

        int nBound1,nBound2;
        int nLoopPx;
        int **ptr1; 
        int tmp_bool;

        const __int64 maskancienrouge = 0x00FFFFFF00FFFFFFLL;
        const __int64 masknouveaurouge= 0xFF000000FF000000LL;
        const __int64 maskancienbleu =  0xFFFF00FFFFFF00FFLL;
        const __int64 masknouveaubleu=  0x0000FF000000FF00LL;
        
        if (m_nHeightBlackBorder>2)
        {
                nBound1=m_nHeightBlackBorder;
                nBound2=CURRENTY-m_nHeightBlackBorder;
        }
        else
        {
                nBound1=0;
                nBound2=CURRENTY-1;
        }

        nLoopPx=m_nSizeBufferX*sizeof(short)/sizeof(__int64);

          __asm__  __volatile__ (
                "movq    %0, %%mm4 \n\t"                           //mask couleur a garder
                "movq    %1, %%mm5 \n\t"                  //mask couleur de remplacement
                "movq    %2, %%mm6 \n\t"                            //mask couleur a garder
                "movq    %3, %%mm7 \n\t"                       //mask couleur de remplacement
                :  :
                 "m" (maskancienrouge), "m" (masknouveaurouge),  "m" (maskancienbleu),  "m" (masknouveaubleu)
                 : "memory"
                 );

        tmp_bool= (int)(!bPhaseFrame) ^ (nBound1&1!=0) ;
        

        ptr1=(int**)m_pnLineSort288;
                
          __asm__ __volatile__ (
                //modif Gondos
                //thanks Gondos !

                "movl            %0, %%eax       \n\t"
                "movl            %1, %%ebx       \n\t"
                "decl            %%eax       \n\t"
                "movl            %2, %%edx       \n\t"
//                "jmp             FS.boucle      \n\t"
                "jmp 0f \n\t"
                ".align 16                   \n\t"

//"FS.boucle:                     \n\t"
                "0: \n\t"
                "movl            (%%edx,%%eax,4), %%esi \n\t"                           //pnLineNear reste dans esi
                "movq    %%mm4, %%mm2        \n\t"                                      //mask couleur a garder rouge

                "incl            %%eax       \n\t"
                "movq    %%mm5, %%mm3        \n\t"                                      //mask couleur de remplacement rouge

                "xorl            $1, %%ebx   \n\t" //if(tmp_bool^=1)
                "movl            (%%edx,%%eax,4), %%edi \n\t"                           //pnLineCurrent reste dans edi

                // "jne             FS.fincas      \n\t"
                "jne 1f \n\t"

                /////////////////////////////////////////
                //      Si il y a plus de bleu que     //
                //      que de  rouge a  corriger,     //
                //      mettre un 'je' et inverser     //
                //      mm6/mm4 et mm7/mm5 pour la     //
                //      branch prediction...           //
                /////////////////////////////////////////

                "movq    %%mm6, %%mm2        \n\t"                                      //mask couleur a garder bleu
                "movq    %%mm7, %%mm3        \n\t"                                      //mask couleur de remplacement bleu

// "FS.fincas:                     \n\t"
                "1: \n\t"
                "movl            %3, %%ecx       \n\t"
//                "jmp             FS.AutrePixel  \n\t"
                "jmp 2f\n\f"
".align 16                   \n\t"
//        "FS.AutrePixel:                 \n\t"
                 "2: \n\t"

                        //read color
                        "movq    (%%edi), %%mm0      \n\t"                      //mm0=pnLineCurrent

                        "movq    (%%esi), %%mm1      \n\t"                      //mm1=pnLineNear
                        //filter
                        "pand    %%mm2, %%mm0        \n\t"                              //mm0=mm0&mm2

                        "addl            $8, %%edi   \n\t"
                        "pand    %%mm3, %%mm1        \n\t"                              //mm0=mm0&mm2

                        "addl            $8, %%esi   \n\t"
                        "por             %%mm1, %%mm0 \n\t"                                     //mm0=mm0|mm1

                        "decl            %%ecx       \n\t"

                        //record
                        "movq    %%mm0, -8(%%edi)    \n\t"                      //pnLineCurrent=mm0
//                        "jnz             FS.AutrePixel  \n\t"                                                      //Sortie
                        "jnz 2b \n\t"

                        "cmpl            %4,%%eax       \n\t"
//                        "jl              FS.boucle      \n\t"
                        "jl 0b \n\t"

                        "emms                        \n\t" // sortie du mode MMX

           : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

           : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
            "m" (nBound1), "m" (tmp_bool), "m" (ptr1), "m" (nLoopPx), "m" (nBound2)
           : "%eax", "%ebx", "%ecx", "%edi", "%esi" // CHECKASM: clobber list
        );

}



void CLaVache::FilterPurple()
{

        int py;
        int * pnLineCurrent;
        int * pnLineNear;
        int nBound1,nBound2;
        int nLoopPx;
        bool bBlockSupPurple=false;
        bool bBlockInfPurple=false;

        //toggle array construction
        for (py=0;py<CURRENTY;py++)
        {
                m_psSortLineReverse[m_psSortLineCurrent[py]]=py;
        }


        nBound1=0;
        nBound2=CURRENTY;
        if (m_nHeightBlackBorder>4)
        {
                nBound1=m_nHeightBlackBorder-4;
                nBound2=CURRENTY-m_nHeightBlackBorder+4; 
        }

        //Saturate lines calcul  --------------
        ZeroMemory(m_pbVioletDetect,sizeof(*m_pbVioletDetect)*CURRENTY);
        ZeroMemory(m_pbVioletSupprim,sizeof(*m_pbVioletSupprim)*CURRENTY);

        bool bImpaireOffset=((m_nKeyFound>>7)&0x01!=0);
        bBlockSupPurple=(((m_bIsOdd==0)^(bImpaireOffset))!=0);
        
        bBlockInfPurple=!bImpaireOffset;

        
        for (py=0;py<22;py++)   //not always purple
                m_pbVioletDetect[m_psSortLineReverse[py]]=DetectLinePurple(m_pnLineCurent288[py]);

        //Block Sup
        if (bBlockSupPurple)
        {
                for (py=32;py<53;py++)  //always purple
                        m_pbVioletDetect[m_psSortLineReverse[py]]=true;
        }

        //Block Inf 
        if (bBlockInfPurple)
        {
                for (py=32;py<CURRENTY;py++)
                {
                        if (m_psSortLineReverse[py]>254)
                        {
                                m_pbVioletDetect[m_psSortLineReverse[py]]=true;
                                if (py+1<CURRENTY)
                                        m_pbVioletDetect[m_psSortLineReverse[py+1]]=true;
                        }
                }
        }


        //Kill saturate lines
        __int64 masklum = 0x00FF00FF00FF00FFLL;
        __int64 maskcoul = 0xFF00FF00FF00FF00LL;
        __int64 maskcoul80 = maskcoul&0x8080808080808080LL;

          __asm__  __volatile__ (
                "movq    %0,    %%mm2    \n\t"
                "movq    %1,   %%mm3   \n\t"
                "movq    %2, %%mm4   \n\t"
                  : :
           "m" (masklum), "m" (maskcoul), "m" (maskcoul80)
                  : "memory"
        );

        nLoopPx=m_nSizeBufferX*sizeof(short)/sizeof(__int64);

        //use near color border of lines
        for (py=nBound1;py<nBound2;py++)
        {
                if ((m_pbVioletDetect[py])&&(py>1)&&(py<CURRENTY-4))
                {
                        if ((!m_pbVioletDetect[py-2])||(!m_pbVioletDetect[py+2]))
                        {
                                m_pbVioletSupprim[py]=true;

                                if ((!m_pbVioletDetect[py-2]))
                                        pnLineNear=m_pnLineSort288[py-2];
                                else
                                        pnLineNear=m_pnLineSort288[py+2];

                                pnLineCurrent=m_pnLineSort288[py];

                                  __asm__  __volatile__ (
                                        //modif Gondos
                                        "movl            %0, %%esi \n\t"
                                        "movl            %1, %%edi \n\t"

                                        "movl            %2, %%ecx  \n\t"
                                        // "FP.AutrePixel_3:            \n\t"
					"0: \n\t"

                                        "movq    (%%esi), %%mm1      \n\t"              //mm1=ligne_precedente

                                        "movq    (%%edi), %%mm0      \n\t"              //mm0=ligne_courante
                                        "pand    %%mm3, %%mm1        \n\t"                      //mm1=mm1&maskcoul

                                        //remplacement de la couleur
                                        "pand    %%mm2, %%mm0        \n\t"                      //mm0=mm0&masklum
                                        "addl            $8, %%edi   \n\t"

                                        "por             %%mm1, %%mm0 \n\t"                             //mm0=mm0|mm1 ajoute la couleur precedente
                                        "addl            $8, %%esi   \n\t"

                                        //stockage
                                        "movq  %%mm0, -8(%%edi)    \n\t"
                                        "decl  %%ecx                  \n\t"

                                        //"jnz FP.AutrePixel_3            \n\t"
                                        "jnz 0b \n\t"


                                   : // FIXASM: output regs/vars go here, e.g.:  "=m" (memory_var)

                                   : // FIXASM: input regs, e.g.:  "c" (count), "S" (src), "D" (dest)
                                    "m" (pnLineNear),  "m" (pnLineCurrent),  "m" (nLoopPx)

                                   : "%ecx", "%edi", "%esi", "memory" // CHECKASM: clobber list

                                );

                        }


                }
        }



        //use previous frame or clear color
        for (py=nBound1;py<nBound2;py++)
        {
                if ((m_pbVioletDetect[py])&&(!m_pbVioletSupprim[py]))
                {
                        pnLineCurrent=m_pnLineSort288[py];

                        if ((m_bOldKeyFound[m_bIsOdd]) && (py<CURRENTY-1))
                        //if (0==1)     ///*!!!!!
                        {       //get color on previous frame
                           pnLineNear=m_pnLineOldSort288[m_bIsOdd][py];

                                __asm__  __volatile__                  // __asm Block
                                (
                                        //modif Gondos
                                        "movl             %0, %%esi \n\t"
                                        "movl             %1, %%edi \n\t"
                                        "movl             %2,%%ecx \n\t"

                                      //  "FP.AutrePixel: \n\t"
                                        "0: \n\t"
                                        "movq    (%%esi),%%mm1 \n\t"   //mm1=ligne_precedente
                                        "movq    (%%edi),%%mm0 \n\t"   //mm0=ligne_courante
                                        "pand    %%mm3,%%mm1 \n\t"   //mm1=mm1&maskcoul

                                        //fill color
                                        "pand    %%mm2,%%mm0  \n\t"   //mm0=mm0&masklum
                                        "add     $8,%%edi  \n\t"
                                        "por     %%mm1,%%mm0  \n\t"  //mm0=mm0|mm1 ajoute la couleur precedente
                                        "add     $8,%%esi  \n\t"

                                        //record
                                        "movq    %%mm0, -8(%%edi) \n\t"
                                        "decl    %%ecx  \n\t"

                                        // "jnz FP.AutrePixel \n\t"
					"jnz 0b \n\t"

                                        :   // output

                                        :   // input
                                           "m" (pnLineNear), "m" (pnLineCurrent), "m" (nLoopPx)
                                        :   "%ecx", "%edi", "%esi", "memory" // clobber
                                 );
                        }
                        else
                        { // clear color

                     __asm__ __volatile__                  // __asm Blockk
                                (

                                        //modif Gondos
                                        "movl             %0, %%edi \n\t"
                                        "movl             %1, %%ecx \n\t"
                                        //"FP.AutrePixel2:   \n\t"
					"1: \n\t"

                                        "movq   (%%edi),%%mm0 \n\t"              //mm0=ligne_courante

                                        // clear color
                                        "pand    %%mm2,%%mm0 \n\t"                                        //mm0=mm0&masklum
                                        "add     $8, %%edi \n\t"

                                        "por     %%mm4,%%mm0 \n\t"       //mm0=mm0|maskcoul80 -> noir et blanc
                                        "decl %%ecx \n\t"

                                        //record
                                        "movq    %%mm0, -8(%%edi) \n\t"
                                        // "jnz FP.AutrePixel2 \n\t"
					"jnz 1b \n\t"

                                     : // output
                                     : // input
                                      "m" (pnLineCurrent),"m" (nLoopPx)
                                     : "%ecx", "%edi", "memory" // clobber

                                );

                        }

                }

        }

        __asm__ __volatile__    ("emms");          //ouput mmx mode

}




 bool CLaVache::DetectLinePurple(int * pnLine)
{
        const int ThresholdSaturation=145;      //150

        int ThresholdPixelMiss=(m_nSizeBufferX/2*20)/100;       // 20%
        int px;
        int nColorCurrent;
        int nSaturationMiss;


        nSaturationMiss=0;
        for (px=0;px<m_nSizeBufferX/2;px++)
        {
                nColorCurrent=*(pnLine+px);
                if (( (unsigned char)(nColorCurrent>>8)<ThresholdSaturation )
                        &&( (unsigned char)(nColorCurrent>>24)<ThresholdSaturation ))
                {       ///no red and no blue
                        nSaturationMiss++;

                        if (nSaturationMiss>ThresholdPixelMiss) //miss nSaturationMiss
                                break;
                }

        }

        if (nSaturationMiss<=ThresholdPixelMiss)
                return true;

        return false;
}


void CLaVache::FindHeightBlackBorder()
{
        const int nBackThresholdMax=60;
        const int nHeightThreshold=15;
        const int nHeightTested=70;
        const int nMaxBadLine=3;

        int nBlackLineBottom;
        int nColum;
        int nBlackLineTop;
        int py;
        int nBadBlackLine=0;

        //top Threshold
        int nBackAverage=0;

         if (!m_bKeyFound)
                return;

        for (py=2;py<nHeightThreshold+2;py++)
        {
                for (nColum=0;nColum<NCOLUMNTESTED;nColum++)
                {
                        nBackAverage+=(unsigned char)(m_pnColumnValue[nColum][m_psSortLineCurrent[py]]);
                }
        }
        nBackAverage=nBackAverage/(NCOLUMNTESTED*nHeightThreshold)+6;
        if (nBackAverage>nBackThresholdMax)
                nBackAverage=nBackThresholdMax;

        //top border 0->nHeightTested
        for (nBlackLineTop=2;nBlackLineTop<nHeightTested;nBlackLineTop++)
        {
                int nValueBlackLine=0;
                for (nColum=0;nColum<NCOLUMNTESTED;nColum++)
                {
                        nValueBlackLine+=(unsigned char)(m_pnColumnValue[nColum][m_psSortLineCurrent[nBlackLineTop]]);
                }
                nValueBlackLine=nValueBlackLine/NCOLUMNTESTED;
                if (nValueBlackLine>nBackAverage)
                        nBadBlackLine++;
                if (nBadBlackLine>=nMaxBadLine)
                        break;
        }
        nBlackLineTop-=nMaxBadLine;


        //bottom border CURRENTY-1->CURRENTY-HeightTested
        nBadBlackLine=0;
        for (nBlackLineBottom=CURRENTY-3;nBlackLineBottom>=CURRENTY-nHeightTested;nBlackLineBottom--)
        {
                int nValueBlackLine=0;
                for (int nColum=0;nColum<NCOLUMNTESTED;nColum++)
                {
                        nValueBlackLine+=(unsigned char)(m_pnColumnValue[nColum][m_psSortLineCurrent[nBlackLineBottom]]);
                }
                nValueBlackLine=nValueBlackLine/NCOLUMNTESTED;
                if (nValueBlackLine>nBackAverage)
                        nBadBlackLine++;
                if (nBadBlackLine>=nMaxBadLine)
                        break;
        }
        nBlackLineBottom+=nMaxBadLine;

        int nNewHeightBlack=min(nBlackLineTop,CURRENTY-nBlackLineBottom);
        if (nNewHeightBlack<10)
                nNewHeightBlack=0;
        
        m_fHeightBlackDynam=(float)((m_fHeightBlackDynam*10.0+nNewHeightBlack)/11.0);
}


int CLaVache::GetHeightBlackDynam()
{
        if (m_fHeightBlackDynam<20)
                return 0;
        else
                return((int)m_fHeightBlackDynam);
}


void CLaVache::SetHeightBlackBorder(int nHeightBlackBorder)
{
        m_nHeightBlackBorder=nHeightBlackBorder;
}


bool CLaVache::CheckFileID(FILE *sFile)
{
        unsigned int i;

        if (!sFile)
                return false;

        char cCheckFileID[sizeof(cFileID)];
        char cCheckFileVersion[sizeof(cFileVersion)];


        if(fread(cCheckFileID,sizeof(cCheckFileID),1,sFile)==0)
                return false;

        if(fread(cCheckFileVersion,sizeof(cCheckFileVersion),1,sFile)==0)
                return false;

        //cCheckFileID
        for (i=0;i<sizeof(cCheckFileID);i++)
        {
                if (cCheckFileID[i]!=cFileID[i])
                        return false;
        }

        //cCheckFileVersion
        for (i=0;i<sizeof(cCheckFileVersion);i++)
        {
                if (cCheckFileVersion[i]!=cFileVersion[i])
                        return false;
        }

        return true;
}

