/*************************************************************************
* channel_gui.c                                                          *
* Maupatz     2001                                                       *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include <Imlib.h>
#include <pthread.h>
#include <string.h>
#include "taztv.h"
#include "video.h"
#include "x11.h"
#include "channel_gui.h"
#include "gui.h"
#include "gui_list.h"
#include "gui_button.h"
#include "gui_labelbutton.h"
#include "gui_checkbox.h"
#include "gui_label.h"
#include "gui_dnd.h"
#include "gui_slider.h"
#include "gui_parseskin.h"
#include "gui_image.h"
#include "gui_browser.h"
#include "i18n.h"

extern Display        *gDisplay;

static pthread_t channel_list_thread;

static int channel_list_gui_running=0;
static int end_channel_list=0;
static Window          chl_win;
static widget_list_t  *ch_widget_list;
static widget_t       *ch_list = NULL;
static char   *tv_channels_name[NB_TV_CHANNEL_MAX];
static ImlibImage     *chl_bg_image;
static char *label_channel_list;

//static int chl_mask_event = ButtonPressMask | ButtonReleaseMask | PointerMotionMask
//	              | KeyPressMask | ExposureMask | StructureNotifyMask;

static int chl_mask_event = ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
	                    ExposureMask;

 /*
 * Leaving channel list
 */

static void channel_list_exit(widget_t *w, void *data) {


  channel_list_gui_running=0;

  XUnmapWindow(gDisplay, chl_win);

  gui_list_free(ch_widget_list->l);
  free(ch_widget_list);
  ch_widget_list = NULL;

  XDestroyWindow(gDisplay, chl_win);
  chl_win = 0;

  if (debug) fprintf(stderr,"channel list unpop\n");

  pthread_exit((void *) 0 );


}

void abort_channel_list_gui(void){
  void *ret;
  if (channel_list_gui_running==0) return;
  end_channel_list=1;
  pthread_join(channel_list_thread, &ret);
}


static void *main_channel_list_loop (void *arg){

   XEvent event[1];
   widget_t *w=NULL;
   void *data=NULL;
//   int k;

 while (!end_channel_list) {
       usleep(10000);
       if (XPending(gDisplay)) {
       while (XCheckWindowEvent (gDisplay,chl_win,chl_mask_event,event)!=False){
             switch(event->type) {
              case Expose:
               paint_widget_list (ch_widget_list);
              break;

              case MotionNotify:
                 motion_notify_widget_list (ch_widget_list,
				 event->xbutton.x, event->xbutton.y);
              break;

              case ButtonPress:
                click_notify_widget_list (ch_widget_list, event->xbutton.x,
			        event->xbutton.y, 0);
              break;

              case ButtonRelease:
                  click_notify_widget_list (ch_widget_list, event->xbutton.x,
				event->xbutton.y, 1);

              break;
            }
       }
//          k=XPending(gDisplay);
//          if (k) { printf ("Reste %d\n",k); } else  printf ("Reste rien\n");
   }
 }

 channel_list_exit (w,data);
 return 0;
}
/*
 * Set channel
 */
static void handle_selection(widget_t *w, void *data) {

//  printf(" +++ Selection called = %d = '%s'\n",
//        ((int)data), tv_channels_name[((int)data)]);

  print_status_bar(set_tv_channel_by_number ((int)data + 1 ));
  end_channel_list=1;
}

int pop_channel_list_gui(int x, int y){

  int *arg=0;
  GC                      gc;
  XSizeHints              hint;
  XWMHints               *wm_hint;
  XSetWindowAttributes    attr;
  int                     screen;
  char                    title[] = {"Taztv channel selection"};
  Atom                    prop;
  MWMHints                mwmhints;
  XClassHint             *xclasshint;
  int i;

  if (nbchaines==0) return -1;
  if (channel_list_gui_running) return -1;

  label_channel_list  = _("Television channel list");

  /*
   * Build channel list
   */
   for (i=0;i<NB_TV_CHANNEL_MAX;i++) tv_channels_name[i] = 0;
   for (i=0;i<nbchaines;i++){
   tv_channels_name[i] = tv_channel_list [i].name;
 //   printf ("%s\n",tv_channels_name[i]);
   }

  if (!(chl_bg_image = Imlib_load_image(gImlib_data,
				       gui_get_skinfile("PlBG")))) {
    if (debug) fprintf(stderr, "xine-playlist: couldn't find image for background\n");
    return -1;
  }

  screen = DefaultScreen(gDisplay);
  hint.x = x - gui_get_skinX("PlItemBtn") - 128;
  hint.y = y - gui_get_skinY("PlItemBtn") - 8 ;
  hint.width = chl_bg_image->rgb_width;
  hint.height = chl_bg_image->rgb_height;
  hint.flags = PPosition | PSize;

  attr.override_redirect = True;
  chl_win = XCreateWindow (gDisplay, DefaultRootWindow(gDisplay),
			  hint.x, hint.y, hint.width, hint.height, 0,
			  CopyFromParent, CopyFromParent,
			  CopyFromParent,
			  0, &attr);

  XSetStandardProperties(gDisplay, chl_win, title, title,
			 None, NULL, 0, &hint);
  /*
   * wm, no border please
   */

  prop = XInternAtom(gDisplay, "_MOTIF_WM_HINTS", True);
  mwmhints.flags = MWM_HINTS_DECORATIONS;
  mwmhints.decorations = 0;

  XChangeProperty(gDisplay, chl_win, prop, prop, 32,
                  PropModeReplace, (unsigned char *) &mwmhints,
                  PROP_MWM_HINTS_ELEMENTS);

  XSetTransientForHint(gDisplay, chl_win, None);

 /* set xclass */
  if((xclasshint = XAllocClassHint()) != NULL) {
    xclasshint->res_name = "Taztv Panel";
    xclasshint->res_class = "Taztv";
    XSetClassHint(gDisplay, chl_win, xclasshint);
    XFree(xclasshint);
  }

  gc = XCreateGC(gDisplay, chl_win, 0, 0);
  XSetGraphicsExposures(gDisplay,gc,False);

  XSelectInput(gDisplay, chl_win,chl_mask_event);

  wm_hint = XAllocWMHints();
  if (wm_hint != NULL) {
    wm_hint->input = True;
    wm_hint->initial_state = NormalState;
    wm_hint->flags = InputHint | StateHint;
    XSetWMHints(gDisplay, chl_win, wm_hint);
    XFree(wm_hint);
  }

  XLOCK();
  Imlib_apply_image(gImlib_data, chl_bg_image, chl_win);

  XSync(gDisplay, False);
  XSync(gDisplay, False);
  
  /*
   * Widget-list
   */
  ch_widget_list = (widget_list_t *) xmalloc (sizeof (widget_list_t));
  ch_widget_list->l = gui_list_new ();
  ch_widget_list->focusedWidget = NULL;
  ch_widget_list->pressedWidget = NULL;
  ch_widget_list->win           = chl_win;
  ch_widget_list->gc            = gc;


  gui_list_append_content (ch_widget_list->l,
			   create_label_button (gui_get_skinX("PlDismiss"),
						gui_get_skinY("PlDismiss"),
						CLICK_BUTTON, _("Dismiss"),
						channel_list_exit, NULL,
						gui_get_skinfile("PlDismiss"),
						gui_get_ncolor("PlDismiss"),
						gui_get_fcolor("PlDismiss"),
						gui_get_ccolor("PlDismiss")));


  gui_list_append_content (ch_widget_list->l,
			   (ch_list = create_browser(ch_widget_list,
						     gui_get_skinX("PlUp"),
						     gui_get_skinY("PlUp"),
						     gui_get_skinfile("PlUp"),
						     gui_get_skinX("PlSlidBG"),
						     gui_get_skinY("PlSlidBG"),
						     gui_get_skinfile("PlSlidBG"),
						     gui_get_skinfile("PlSlidFG"),
						     gui_get_skinX("PlDn"),
						     gui_get_skinY("PlDn"),
						     gui_get_skinfile("PlDn"),
						     gui_get_skinX("PlItemBtn"),
						     gui_get_skinY("PlItemBtn"),
						     gui_get_ncolor("PlItemBtn"),
						     gui_get_fcolor("PlItemBtn"),
						     gui_get_ccolor("PlItemBtn"),
						     gui_get_skinfile("PlItemBtn"),
						     9, nbchaines, tv_channels_name,
						     handle_selection, NULL)));

  gui_list_append_content (ch_widget_list->l,
			   create_label (gui_get_skinX("AutoPlayLbl"),
					 gui_get_skinY("AutoPlayLbl"),
					 (int) strlen(label_channel_list),
                                          label_channel_list,
					 gui_get_skinfile("AutoPlayLbl")));

  gui_list_append_content (ch_widget_list->l,
			   create_image (gui_get_skinX("tazch"),
					 gui_get_skinY("tazch"),
					 gui_get_skinfile("tazch")));


   browser_update_list(ch_list, tv_channels_name, nbchaines , 0);

   XMapRaised(gDisplay, chl_win);

   XSync(gDisplay, False);
   XSync(gDisplay, False);
   XUNLOCK();

   end_channel_list=0;
   channel_list_gui_running=1;
   pthread_create(&channel_list_thread, NULL, main_channel_list_loop ,(void *) arg);
   if (debug) fprintf(stderr,"channel list pop\n");
  return 0;
}





