/*************************************************************************
* decode_gui.c                                                             *
* Maupatz     2001                                                       *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include "taztv.h"
#include "video.h"
#include "alevt.h"
#include "readconfig.h"
#include "decode_gui.h"
#include <pthread.h>
#include "gui.h"
#include "gui_list.h"
#include "gui_button.h"
#include "gui_labelbutton.h"
#include "gui_checkbox.h"
#include "gui_label.h"
#include "gui_dnd.h"
#include "gui_slider.h"
#include "gui_parseskin.h"
#include "gui_image.h"
#include "gui_browser.h"
#include "i18n.h"
#include "nagra.h"
#include "mixer.h"
#include "kizkool.h"


static Window          decode_win;
static widget_list_t  *decode_widget_list;
static ImlibImage     *decode_bg_image;

static int datachanged = 0;

static widget_t  *SoundInputLbl;
static int  SoundInputVal;
static char *SoundInputcharVal;

static widget_t  *DecSLbl;
static int DecSVal;
static char *DecScharVal;

static widget_t  *DecLbl;
static int DecVal;
static char *DeccharVal;

static widget_t  *CouplesLbl;
static int CouplesVal;
static char *CouplescharVal;

static widget_t  *QualityLbl;
static int QualityVal;
static char *QualitycharVal;

static widget_t  *FilterLbl;
static int FilterVal;
static char *FiltercharVal;

static widget_t  *UValLbl;
static int UVal;
static char UcharVal[4];

static widget_t  *VValLbl;
static int VVal;
static char VcharVal[4];

static widget_t  *YUVValLbl;
static int YUVVal;
static char YUVcharVal[4];

static widget_t  *PIXValLbl;
static int PIXVal;
static char PIXcharVal[4];



static int decode_mask_event = ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
	                     ExposureMask;

static int end_decode_gui=0;
static int decode_gui_running=0;
static pthread_t decode_gui_thread;

/* decode gui do */

#define NEXTCH    1
#define PREVCH    2

/*
   Sound decoder
   decode : on off auto
   Input
*/

/*
   Image decoder
   decode : on off auto
   couples
   quality
   filter
   uthreshold
   vthreshold
   yuvthreshold
   pixthreshold
*/


static void dec_ch_sound  (widget_t *w, void *data ) {
 SEND_KEVENT(XK_S);
}

static void dec_ch_input_sound (widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH)
  {
   SoundInputVal++;
   if (rec_sound_names[SoundInputVal]==NULL)
    {
     SoundInputVal--;
     return;
    }
    else
    {
     set_mixer_input(rec_sound_names[SoundInputVal]);
    }
  } else if ( data == (void *) PREVCH)
   {
    SoundInputVal--;
    if (SoundInputVal<0)
    {
     SoundInputVal=0;
     return;
    }
    else
    {
     set_mixer_input(rec_sound_names[SoundInputVal]);
    }
   }

  SoundInputcharVal=get_mixer_input_name();
  label_change_label (decode_widget_list,SoundInputLbl, SoundInputcharVal);
  datachanged = 1;

}

static void dec_ch_decode (widget_t *w, void *data ) {
 SEND_KEVENT(XK_D);
}

static void dec_ch_couples (widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH)
  {
   SEND_KEVENT(XK_Insert);
  } else

  if ( data == (void *)PREVCH)
  {
   SEND_KEVENT(XK_Delete);
  }
}

static void dec_ch_quality (widget_t *w, void *data ){
  SEND_KEVENT(XK_I);
}

static void dec_ch_filter(widget_t *w, void *data ){
 SEND_KEVENT(XK_N);
}

static void set_u(widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH)
  {
   UVal++;
   if (UVal>100) UVal=100;
  } else if ( data == (void *) PREVCH)
         {
          UVal--;
          if (UVal<0) UVal=0;
         }
  setfilterparam( KIZKOOL_UTHRESHOLD,UVal );
  sprintf(UcharVal,"%d",UVal);
  label_change_label (decode_widget_list,UValLbl,UcharVal);
}

static void set_v(widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH)
  {
   VVal++;
   if (VVal>100) VVal=100;
  } else if ( data == (void *) PREVCH)
         {
          VVal--;
          if (VVal<0) VVal=0;
         }
 setfilterparam( KIZKOOL_VTHRESHOLD,VVal );
 sprintf(VcharVal,"%d",VVal);
 label_change_label (decode_widget_list,VValLbl,VcharVal);
}

static void set_yuv(widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH)
  {
   YUVVal++;
   if (YUVVal>100) YUVVal=100;
  } else if ( data == (void *) PREVCH)
         {
          YUVVal--;
          if (YUVVal<0) YUVVal=0;
         }
 setfilterparam( KIZKOOL_YUVTHRESHOLD,YUVVal );
 sprintf(YUVcharVal,"%d",YUVVal);
 label_change_label (decode_widget_list,YUVValLbl,YUVcharVal);
}

static void set_pix(widget_t *w, void *data ) {

  if ( data == (void *) NEXTCH)
  {
   PIXVal++;
   if (PIXVal>100) PIXVal=100;
  } else if ( data == (void *) PREVCH)
         {
          PIXVal--;
          if (PIXVal<0) PIXVal=0;
         }
 setfilterparam( KIZKOOL_PIXFILTERTHRESHOLD,PIXVal);
 sprintf(PIXcharVal,"%d",PIXVal);
 label_change_label (decode_widget_list,PIXValLbl,PIXcharVal);
}

/**********************************************************/

 /*
 * Leaving decode_gui
 */

static void decode_gui_exit(widget_t *w, void *data) {

  decode_gui_running=0;

  XUnmapWindow(gDisplay, decode_win);

  gui_list_free(decode_widget_list->l);
  free(decode_widget_list);
  decode_widget_list = NULL;

  XDestroyWindow(gDisplay, decode_win);
  decode_win = 0;

  if (debug) fprintf(stderr,"Decode gui stop\n");

  pthread_exit((void *) 0 );

}

static void decode_gui_close(widget_t *w, void *data ) {
  end_decode_gui = 1;
}


static void *decode_gui_loop (void *arg){

   XEvent event[1];
   widget_t *w=NULL;
   void *data=NULL;
   int x=0,y=0,ox=0,oy=0;
   int move_win=0;
   Window wr;
   int k;

 while (!end_decode_gui) {

      /* Thread Method is wonderful */
      /* Test des variables         */

       usleep(10000);

       get_sound_decoder_mode(&k);
       if (k!=DecSVal)
       {
         DecSVal=k;
         DecScharVal=get_char_sound_decoder_mode();
         label_change_label (decode_widget_list,DecSLbl,DecScharVal);
       }

      if (!disable_decoding)
      {
       get_decoder_mode(&k);
       if (k!=DecVal)
       {
         DecVal=k;
         DeccharVal=get_char_decoder_mode();
         label_change_label (decode_widget_list,DecLbl,DeccharVal);
       }
       get_quality_mode(&k);
       if (k!=QualityVal)
       {
         QualityVal=k;
         QualitycharVal=get_char_quality_mode();
         label_change_label (decode_widget_list,QualityLbl,QualitycharVal);
       }
       get_filter_mode(&k);
       if (k!=FilterVal)
       {
         FilterVal=k;
         FiltercharVal=get_char_filter_mode();
         label_change_label (decode_widget_list,FilterLbl,FiltercharVal);
       }
       get_couples(&k);
       if (k!=CouplesVal)
       {
         CouplesVal=k;
         CouplescharVal=char_get_couples();
         label_change_label (decode_widget_list,CouplesLbl,CouplescharVal);
       }
      }

       if (XPending(gDisplay)) {

       while (XCheckWindowEvent (gDisplay,decode_win,decode_mask_event,event)!=False){
             switch(event->type) {
              case Expose:
               paint_widget_list (decode_widget_list);
              break;

              case MotionNotify:
                 if (move_win) {
                                  XLOCK();
                                  XTranslateCoordinates(gDisplay,decode_win,
                                  RootWindow(gDisplay,DefaultScreen(gDisplay)),
                                  0,0,&x,&y,&wr);
                                  x = x + event->xmotion.x - ox;
                                  y = y + event->xmotion.y - oy;
                                  XMoveWindow(gDisplay, decode_win, x, y);
                                  XSync(gDisplay, False);
                                  XSync(gDisplay, False);
  /* let's go faster :-) */
 while (XCheckWindowEvent (gDisplay,decode_win,PointerMotionMask,event)==True);
                                  XUNLOCK();

                           /* fin */
    //                   printf ("%d %d %d %d\n",event->xmotion.x_root, event->xmotion.y_root,
    //                     event->xmotion.x, event->xmotion.y );
                 } else
                 motion_notify_widget_list (decode_widget_list,
	  			 event->xbutton.x, event->xbutton.y);
              break;

              case ButtonPress:
                move_win =
                !click_notify_widget_list (decode_widget_list, event->xbutton.x,
			        event->xbutton.y, 0);
                 if (move_win) { ox = event->xbutton.x; oy =  event->xbutton.y; }
              break;

              case ButtonRelease:
                  move_win = 0;
                  click_notify_widget_list (decode_widget_list, event->xbutton.x,
				event->xbutton.y, 1);

              break;
          /*
              case ConfigureNotify:
                Imlib_apply_image(gImlib_data, decode_bg_image,decode_win );
                paint_widget_list (decode_widget_list);
                XSync(gDisplay, False);
              break;
           */
            }
       }
//          k=XPending(gDisplay);
//          if (k) { printf ("Reste %d\n",k); } else  printf ("Reste rien\n");
   }
 }

 /* Data to config file */
 if (datachanged) {

     open_config_file();
     save_config_value("sound_input",SoundInputcharVal,AT_BEGIN );
     close_config_file();
 
 }

 decode_gui_exit (w,data);
 return 0;
}

int start_decode_gui(void){

  int *arg=0;
  GC                      gc;
  XSizeHints              hint;
  XWMHints               *wm_hint;
  XSetWindowAttributes    attr;
  int                     screen;
  char                    title[] = {"Taztv Decode window"};
  char                    *decode_gui_title;
  Atom                    prop;
  MWMHints                mwmhints;
  XClassHint             *xclasshint;

  if (decode_gui_running) return -1;

  datachanged = 0;

  decode_gui_title  = _("  Taztv Decode Panel  ");

  if (!(decode_bg_image = Imlib_load_image(gImlib_data,
				       gui_get_skinfile("DecodeBG")))) {
    if (debug) fprintf(stderr, "Decode gui: couldn't find image for background\n");
    return -1;
  }

  screen = DefaultScreen(gDisplay);
  hint.x = decode_bg_image->rgb_width  + 1;
  hint.y = 0;
  hint.width = decode_bg_image->rgb_width;
  hint.height = decode_bg_image->rgb_height;
  hint.flags = PSize; // PPosition | PSize;

  attr.background_pixmap=None;
  attr.background_pixel=0;
  attr.border_pixmap=CopyFromParent;
  attr.border_pixel=0;
  attr.bit_gravity=ForgetGravity;
  attr.win_gravity=NorthWestGravity;
  attr.backing_store=0;
  attr.backing_planes=0;
  attr.backing_pixel=0;
  attr.save_under=False;
  attr.event_mask=0;
  attr.do_not_propagate_mask=0;
  attr.override_redirect=True;
  attr.colormap=CopyFromParent;
  attr.cursor=None;

  decode_win = XCreateWindow (gDisplay, DefaultRootWindow(gDisplay),
			  hint.x, hint.y, hint.width, hint.height, 0,
			  CopyFromParent, CopyFromParent,
			  CopyFromParent,
			  0, &attr);

  XSetStandardProperties(gDisplay, decode_win, title, title,
			 None, NULL, 0, &hint);
  /*
   * wm, no border please
   */

  prop = XInternAtom(gDisplay, "_MOTIF_WM_HINTS", True);
  mwmhints.flags = MWM_HINTS_DECORATIONS;
  mwmhints.decorations = 0;

  XChangeProperty(gDisplay, decode_win, prop, prop, 32,
                  PropModeReplace, (unsigned char *) &mwmhints,
                  PROP_MWM_HINTS_ELEMENTS);

  XSetTransientForHint(gDisplay, decode_win, None);

/* set xclass */
  if((xclasshint = XAllocClassHint()) != NULL) {
    xclasshint->res_name = "Taztv Panel";
    xclasshint->res_class = "Taztv";
    XSetClassHint(gDisplay, decode_win, xclasshint);
    XFree(xclasshint);
  }

  gc = XCreateGC(gDisplay, decode_win, 0, 0);
  XSetGraphicsExposures(gDisplay,gc,False);

  XSelectInput(gDisplay, decode_win, decode_mask_event);

  wm_hint = XAllocWMHints();
  if (wm_hint != NULL) {
    wm_hint->input = True;
    wm_hint->initial_state = NormalState;
    wm_hint->flags = InputHint | StateHint;
    XSetWMHints(gDisplay, decode_win, wm_hint);
    XFree(wm_hint);
  }

  XLOCK();

  Imlib_apply_image(gImlib_data, decode_bg_image, decode_win);

  XSync(gDisplay, False);
  XSync(gDisplay, False);

  /*
   * Widget-list
   */
  decode_widget_list = (widget_list_t *) malloc (sizeof (widget_list_t));
  decode_widget_list->l = gui_list_new ();
  decode_widget_list->focusedWidget = NULL;
  decode_widget_list->pressedWidget = NULL;
  decode_widget_list->win           = decode_win;
  decode_widget_list->gc            = gc;

/**************** Init Value *******************************/
 /*
   Sound decoder
   decode : on off auto
   Input
*/

/*
   Image decoder
   decode : on off auto
   couples
   quality
   filter
   uthreshold
   vthreshold
   yuvthreshold
   pixthreshold
*/

    /* image */
  gui_list_append_content (decode_widget_list->l,
			   create_image (gui_get_skinX("TazDec"),
					 gui_get_skinY("TazDec"),
					 gui_get_skinfile("TazDec")));

  /* Close window */
  gui_list_append_content (decode_widget_list->l,
			   create_label_button (gui_get_skinX("DecDismiss"),
						gui_get_skinY("DecDismiss"),
						CLICK_BUTTON, _("Dismiss"),
						decode_gui_close, NULL,
						gui_get_skinfile("DecDismiss"),
						gui_get_ncolor("DecDismiss"),
						gui_get_fcolor("DecDismiss"),
						gui_get_ccolor("DecDismiss")));

   /* decode label */
   gui_list_append_content (decode_widget_list->l,
			   create_label (gui_get_skinX("DecTvLbl"),
					 gui_get_skinY("DecTvLbl"),
					 (int) strlen(decode_gui_title),
                                          decode_gui_title,
				     	  gui_get_skinfile("DecTvLbl")));


  /* decode sound : on off auto state (5 characters) + arrowup + arrowdown */

    gui_list_append_content (decode_widget_list->l,
			   create_label_button (gui_get_skinX("DecSLbl"),
						gui_get_skinY("DecSLbl"),
						CLICK_BUTTON, _("Sound"),
						dec_ch_sound, NULL,
						gui_get_skinfile("DecSLbl"),
						gui_get_ncolor("DecSLbl"),
						gui_get_fcolor("DecSLbl"),
						gui_get_ccolor("DecSLbl")));

   get_sound_decoder_mode(&DecSVal);
   DecScharVal=get_char_sound_decoder_mode();
   gui_list_append_content (decode_widget_list->l,
			   DecSLbl = create_label (gui_get_skinX("DecSVal"),
					 gui_get_skinY("DecSVal"),
					 4,
                                         DecScharVal,
					 gui_get_skinfile("DecSVal")));

 /* Input sound */

   gui_list_append_content (decode_widget_list->l,
			   create_label (gui_get_skinX("DecSInputLbl"),
					 gui_get_skinY("DecSInputLbl"),
					 8,
                                         _("Input"),
					 gui_get_skinfile("DecSInputLbl")));

    SoundInputcharVal=get_mixer_input_name();
    for (SoundInputVal=0;rec_sound_names[SoundInputVal]!=NULL;SoundInputVal++)
    {
     if (strcmp(SoundInputcharVal,rec_sound_names[SoundInputVal])==0) break;
    }
    gui_list_append_content (decode_widget_list->l,
	                     SoundInputLbl = create_label (gui_get_skinX("DecSInputVal"),
					 gui_get_skinY("DecSInputVal"),
					 16,
                                         SoundInputcharVal,
					 gui_get_skinfile("DecSInputVal")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecSInputNext"),
				          gui_get_skinY("DecSInputNext"),
					  dec_ch_input_sound,(void*)NEXTCH ,
					  gui_get_skinfile("DecSInputNext")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecSInputPrev"),
					  gui_get_skinY("DecSInputPrev"),
					  dec_ch_input_sound, (void*)PREVCH,
                                          gui_get_skinfile("DecSInputPrev")));




if (!disable_decoding)
{
  /* decode : on off auto */
  gui_list_append_content (decode_widget_list->l,
			   create_label_button (gui_get_skinX("DecLbl"),
						gui_get_skinY("DecLbl"),
						CLICK_BUTTON, _("Image"),
						dec_ch_decode, NULL,
						gui_get_skinfile("DecLbl"),
						gui_get_ncolor("DecLbl"),
						gui_get_fcolor("DecLbl"),
						gui_get_ccolor("DecLbl")));
   get_decoder_mode(&DecVal);
   DeccharVal=get_char_decoder_mode();
   gui_list_append_content (decode_widget_list->l,
			        DecLbl = create_label (gui_get_skinX("DecVal"),
					 gui_get_skinY("DecVal"),
					 4,
                                         DeccharVal,
					 gui_get_skinfile("DecVal")));

  /* couples  */
   gui_list_append_content (decode_widget_list->l,
			   create_label (gui_get_skinX("CouplesLbl"),
					 gui_get_skinY("CouplesLbl"),
					 10,
                                         _("Couples"),
					 gui_get_skinfile("CouplesLbl")));
   get_couples(&CouplesVal);
   CouplescharVal=char_get_couples();
   gui_list_append_content (decode_widget_list->l,
			   CouplesLbl = create_label (gui_get_skinX("CouplesVal"),
					 gui_get_skinY("CouplesVal"),
					 4,
                                         CouplescharVal,
					 gui_get_skinfile("CouplesVal")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("CouplesValNext"),
				          gui_get_skinY("CouplesValNext"),
					  dec_ch_couples,(void*)NEXTCH ,
					  gui_get_skinfile("CouplesValNext")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("CouplesValPrev"),
					  gui_get_skinY("CouplesValPrev"),
					  dec_ch_couples, (void*)PREVCH,
					  gui_get_skinfile("CouplesValPrev")));

  /* quality  */
   gui_list_append_content (decode_widget_list->l,
			   create_label_button (gui_get_skinX("QualityLbl"),
						gui_get_skinY("QualityLbl"),
						CLICK_BUTTON, _("Quality"),
						dec_ch_quality, NULL,
						gui_get_skinfile("QualityLbl"),
						gui_get_ncolor("QualityLbl"),
						gui_get_fcolor("QualityLbl"),
						gui_get_ccolor("QualityLbl")));

   get_quality_mode(&QualityVal);
   QualitycharVal=get_char_quality_mode();
   gui_list_append_content (decode_widget_list->l,
			   QualityLbl = create_label (gui_get_skinX("QualityVal"),
					 gui_get_skinY("QualityVal"),
					 4,
                                         QualitycharVal,
					 gui_get_skinfile("QualityVal")));

  /* filter   */
  gui_list_append_content (decode_widget_list->l,
			   create_label_button (gui_get_skinX("FilterLbl"),
						gui_get_skinY("FilterLbl"),
						CLICK_BUTTON, _("Filter"),
						dec_ch_filter, NULL,
						gui_get_skinfile("FilterLbl"),
						gui_get_ncolor("FilterLbl"),
						gui_get_fcolor("FilterLbl"),
						gui_get_ccolor("FilterLbl")));
   get_filter_mode (&FilterVal);
   FiltercharVal=get_char_filter_mode ();
   gui_list_append_content (decode_widget_list->l,
			   FilterLbl = create_label (gui_get_skinX("FilterVal"),
					 gui_get_skinY("FilterVal"),
					 4,
                                         FiltercharVal,
                                         gui_get_skinfile("FilterVal")));


 /* uthreshold */
   getfilterparam( KIZKOOL_UTHRESHOLD,&UVal );
   sprintf(UcharVal,"%d",UVal);
   gui_list_append_content (decode_widget_list->l,
			   create_label (gui_get_skinX("DecULbl"),
					 gui_get_skinY("DecULbl"),
					 3,
                                         " U ",
					 gui_get_skinfile("DecULbl")));

   gui_list_append_content (decode_widget_list->l,
			   UValLbl = create_label (gui_get_skinX("DecUVal"),
					 gui_get_skinY("DecUVal"),
					 3,
                                         UcharVal,
					 gui_get_skinfile("DecUVal")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecUNext"),
				          gui_get_skinY("DecUNext"),
					  set_u,(void*)NEXTCH ,
					  gui_get_skinfile("DecUNext")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecUPrev"),
					  gui_get_skinY("DecUPrev"),
					  set_u, (void*)PREVCH,
					  gui_get_skinfile("DecUPrev")));
 /* vthreshold  */
   getfilterparam( KIZKOOL_VTHRESHOLD,&VVal );
   sprintf(VcharVal,"%d",VVal);
   gui_list_append_content (decode_widget_list->l,
			   create_label (gui_get_skinX("DecVLbl"),
					 gui_get_skinY("DecVLbl"),
					 3,
                                         " V ",
					 gui_get_skinfile("DecVLbl")));

    gui_list_append_content (decode_widget_list->l,
			   VValLbl = create_label (gui_get_skinX("DecVVal"),
					 gui_get_skinY("DecVVal"),
					 3,
                                         VcharVal,
					 gui_get_skinfile("DecVVal")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecVNext"),
				          gui_get_skinY("DecVNext"),
					  set_v,(void*)NEXTCH ,
					  gui_get_skinfile("DecVNext")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecVPrev"),
					  gui_get_skinY("DecVPrev"),
					  set_v, (void*)PREVCH,
					  gui_get_skinfile("DecVPrev")));
 /* yuvthreshold  */
   getfilterparam( KIZKOOL_YUVTHRESHOLD,&YUVVal );
   sprintf(YUVcharVal,"%d",YUVVal);
   gui_list_append_content (decode_widget_list->l,
			                 create_label (gui_get_skinX("DecYUVLbl"),
					 gui_get_skinY("DecYUVLbl"),
					 3,
                                         "YUV",
					 gui_get_skinfile("DecYUVLbl")));

   gui_list_append_content (decode_widget_list->l,
			   YUVValLbl =  create_label (gui_get_skinX("DecYUVVal"),
					 gui_get_skinY("DecYUVVal"),
					 3,
                                         YUVcharVal,
					 gui_get_skinfile("DecYUVVal")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecYUVNext"),
				          gui_get_skinY("DecYUVNext"),
					  set_yuv,(void*)NEXTCH ,
					  gui_get_skinfile("DecYUVNext")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecYUVPrev"),
					  gui_get_skinY("DecYUVPrev"),
					  set_yuv, (void*)PREVCH,
					  gui_get_skinfile("DecYUVPrev")));
 /* pixthreshold */
   getfilterparam( KIZKOOL_PIXFILTERTHRESHOLD,&PIXVal );
   sprintf(PIXcharVal,"%d",PIXVal);
   gui_list_append_content (decode_widget_list->l,
			    create_label (gui_get_skinX("DecPixLbl"),
					 gui_get_skinY("DecPixLbl"),
					 3,
                                         "PIX",
					 gui_get_skinfile("DecPixLbl")));

    gui_list_append_content (decode_widget_list->l,
			  PIXValLbl = create_label (gui_get_skinX("DecPixVal"),
					 gui_get_skinY("DecPixVal"),
					 3,
                                         PIXcharVal,
					 gui_get_skinfile("DecPixVal")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecPixNext"),
				          gui_get_skinY("DecPixNext"),
					  set_pix,(void*)NEXTCH ,
					  gui_get_skinfile("DecPixNext")));

    gui_list_append_content (decode_widget_list->l,
			   create_button (gui_get_skinX("DecPixPrev"),
					  gui_get_skinY("DecPixPrev"),
					  set_pix, (void*)PREVCH,
					  gui_get_skinfile("DecPixPrev")));

}


  XMapRaised(gDisplay, decode_win);

  XSync(gDisplay, False);
  XSync(gDisplay, False);
  XUNLOCK();

  decode_gui_running=1;
  end_decode_gui=0;
  pthread_create(&decode_gui_thread, NULL, decode_gui_loop ,(void *) arg);

  if (debug) fprintf (stderr,"Decode gui start \n");
  return 0;
}

void abort_decode_gui(void){
  void *ret;
  if (decode_gui_running==0) return;
  end_decode_gui = 1;
  pthread_join(decode_gui_thread, &ret);
  return;
}

void toggle_decode_gui(void) {

  if (decode_gui_running) abort_decode_gui(); else  start_decode_gui();

}


