#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>

#include "config.h"

#ifdef HAVE_LIBLIRC_CLIENT
# include <lirc/lirc_client.h>
#endif
#include "taztv.h"
#include "gui.h"
#include "x11.h"
#include "lirc.h"

/*-----------------------------------------------------------------------*/


#ifdef HAVE_LIBLIRC_CLIENT
static struct lirc_config *config;
static int lirc=0;
#endif

char**
split_cmdline(char *line, int *count)
{
    static char cmdline[1024];
    static char *argv[32];
    int  argc,i;

    strcpy(cmdline,line);
    for (argc=0, i=0; argc<31;)
    {
	argv[argc++] = cmdline+i;
	while (cmdline[i] != ' ' &&
	       cmdline[i] != '\t' &&
	       cmdline[i] != '\0')
	    i++;
	if (cmdline[i] == '\0')
	    break;
	cmdline[i++] = '\0';
	while (cmdline[i] == ' ' ||
	       cmdline[i] == '\t')
	    i++;
	if (cmdline[i] == '\0')
	    break;
    }
    argv[argc] = NULL;

    *count = argc;
    return argv;
}

int lirc_tv_init()
{
#ifdef HAVE_LIBLIRC_CLIENT
    int fd;
    lirc = 0;
    if (debug) fprintf(stderr,"Lirc: init\n");
    if ((fd=lirc_init("taztv",debug))==-1)
    {
      if (debug)fprintf(stderr,"Lirc: no infrared remote support available\n");
      return -1;
    }

    if (0 != lirc_readconfig(NULL,&config,NULL))
    {
        if (debug) fprintf(stderr,"Lirc: Can't config file read \n");
	lirc_deinit();
	return -1;
    }
    fcntl(fd,F_SETFL,O_NONBLOCK);
    fcntl(fd,F_SETFD,FD_CLOEXEC);
    lirc = 1;
    return fd;
#else
    return -1;
#endif
}

int lirc_tv_havedata()
{
#ifdef HAVE_LIBLIRC_CLIENT
    char *code,*cmd,**argv;
    int argc;
    int ret=-1;

    if (!lirc) return -1;

    while (lirc_nextcode(&code)==0)
    {
       if (code==NULL) break;

	while ((ret=lirc_code2char(config,code,&cmd))==0 && cmd!=NULL)
        {
	    if (debug) fprintf(stderr,"lirc: \"%s\"\n", cmd);
            argv = split_cmdline(cmd,&argc);
            do_command(argc,argv);
	}
	free(code);
        if(ret==-1) break;
    }

  /*  if (ret==-1)
    {
      if (debug) fprintf(stderr,"Lirc: Connexion lost \n");
      lirc_tv_deinit();
    }
 */
    return ret;
#else
    return 0;
#endif
}

int lirc_tv_deinit()
{
#ifdef HAVE_LIBLIRC_CLIENT
 if (lirc)
 {
  if (debug) fprintf(stderr,"Lirc: end\n");
  lirc_freeconfig(config);
  lirc_deinit();
  lirc=0;
 }
#endif
 return 0;

}

void do_command(int argc, char **argv)
{

 int i;

 if (argc<=0)
 {
   if (debug)printf("lirc: no argument\n");
   return;
  }

 if (argc==1)
 {
  if (strcmp(argv[0],"quit")==0) SEND_KEVENT(XK_Q);
  return;
 }

 if (argc==2)
 {

  if (strcmp(argv[0],"setstation")==0)
  {
   if (strcmp(argv[1],"next")==0)
   {
    SEND_KEVENT(XK_Right);
    return;
   }

   if (strcmp(argv[1],"prev")==0)
   {
    SEND_KEVENT(XK_Left);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"volume")==0)
  {
   if (strcmp(argv[1],"inc")==0)
   {
    SEND_KEVENT(XK_Up);
    return;
   }
   if (strcmp(argv[1],"dec")==0)
   {
    SEND_KEVENT(XK_Down);
    return;
   }

   if (strcmp(argv[1],"mute")==0)
   {
    SEND_KEVENT(XK_M);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"image_ratio")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    SEND_KEVENT(XK_C);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"decode")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    SEND_KEVENT(XK_D);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"quality")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    SEND_KEVENT(XK_I);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"filter")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    SEND_KEVENT(XK_N);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"sound")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    SEND_KEVENT(XK_S);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"fullscreen")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    SEND_KEVENT(XK_F);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"couples")==0)
  {
   if (strcmp(argv[1],"inc")==0)
   {
    SEND_KEVENT(XK_Insert);
    return;
   }
   if (strcmp(argv[1],"dec")==0)
   {
    SEND_KEVENT(XK_Delete);
    return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"keypad")==0)
  {
   if(sscanf(argv[1],"%d",&i)==1)
   {
     switch (i)
     {
      case 0: SEND_KEVENT(XK_KP_0);
             return;
      case 1: SEND_KEVENT(XK_KP_1);
             return;
      case 2: SEND_KEVENT(XK_KP_2);
             return;
      case 3: SEND_KEVENT(XK_KP_3);
             return;
      case 4: SEND_KEVENT(XK_KP_4);
              return;
      case 5: SEND_KEVENT(XK_KP_5);
             return;
      case 6: SEND_KEVENT(XK_KP_6);
             return;
      case 7: SEND_KEVENT(XK_KP_7);
             return;
      case 8: SEND_KEVENT(XK_KP_8);
             return;
      case 9: SEND_KEVENT(XK_KP_9);
             return;
      default:
      if (debug) printf("Lirc: Invalid number %s %s\n",argv[0],argv[1]);
     }
     return;
   }
   goto notfind;
  }

  if (strcmp(argv[0],"number")==0)
  {
   if (strcmp(argv[1],"toggle")==0)
   {
    if(get_number_mode())
    {
     SEND_KEVENT(XK_KP_Subtract);
    }
    else
    {
     SEND_KEVENT(XK_KP_Add);
    }
    return;
   }
  }
  notfind:;
  if (debug) printf("Lirc: unknow comand %s %s\n",argv[0],argv[1]);
  return;
 }
 if (debug)
 {
   printf("Lirc: Two many arguments maximun 2\n");
   for (i=0;i<argc;i++) printf("%s ",argv[i]);
   printf("\n");

 }
 return;
}







