/*************************************************************************
* mixer.c                                                                *
* Maupatz     2001                                                       *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include "taztv.h"
#include <sys/soundcard.h>
#include "mixer.h"

static char mixer_name[32] = "/dev/mixer";
static char *sound_names[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_NAMES;

char *rec_sound_names[SOUND_MIXER_NRDEVICES+1];

 /* {"none"} SOUND_DEVICE_NAMES;  */

static int devmask = 0, recmask = 0, recsrc = 0;

static int mixerfd=-1;
static int mixer_disable=0;

static int  dev_input;
static int  dev_level_input;
static char dev_input_name[32]="line";

static int level_input;
static int old_pcm_level;
static int dev_pcm=SOUND_MIXER_PCM;
static int record_sound=0;


static int rec_level_input=0;
static int dev_rec_input_gain=-1;
static int recording_level=-1;

static int open_mixer(void)
{

 mixerfd = open(mixer_name, O_RDWR);
 if (mixerfd > 0) return 0;

 mixerfd = open("/dev/mixer", O_RDWR);
 if (mixerfd > 0){ strcpy(mixer_name,"/dev/mixer"); return 0; }
 
 mixerfd = open("/dev/sound/mixer", O_RDWR);
 if (mixerfd > 0) { strcpy(mixer_name,"/dev/sound/mixer"); return 0; }

 if (debug) fprintf (stderr,"Cannot open mixer in init %s \n",mixer_name);
 mixer_disable=1;
 mixerfd=-1;
 return -1;
}

static int close_mixer(void)
{
 if (mixerfd==-1) return -1;
 close(mixerfd);
 mixerfd=-1;
 return 0;
}


int init_mixer(char *device_name){
    int i,k;

    if (device_name!=NULL && *device_name!=0)
    {
      strcpy(mixer_name,device_name);
    }

    if (open_mixer()==-1) return -1;

    if (ioctl(mixerfd, SOUND_MIXER_READ_DEVMASK, &devmask) == -1)
    {
      if (debug) fprintf (stderr,"Cannot recmask\n");
      mixer_disable=1;
      close_mixer();
      return -1;
    }

    if (debug) fprintf (stderr,"Valid devices \n");
    for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
        if ((1 << i) & devmask)
        {
           if (debug) fprintf (stderr,"%s  ",sound_names[i]);
/* Cherche quelque chose comme igain ou rec ou mix */
/* Il s'agit de rgler le niveau d'enregistrement en entre */
/* Il ne doit pas en avoir plus d'1 sur une carte son ? */
           if (i==SOUND_MIXER_IMIX) dev_rec_input_gain = i;
           if (i==SOUND_MIXER_RECLEV) dev_rec_input_gain = i;
           if (i==SOUND_MIXER_IGAIN) dev_rec_input_gain = i;
      }

    if (debug) fprintf (stderr,"\n");
    if (debug && dev_rec_input_gain!=-1 )
        fprintf (stderr,"Use %s to set the recording level\n",sound_names[dev_rec_input_gain]);

    if (ioctl(mixerfd, SOUND_MIXER_READ_RECMASK, &recmask) == -1)
    {
      if (debug) fprintf (stderr,"Cannot recmask\n");
      mixer_disable=1;
      close_mixer();
      return -1;
    }

/* sauvergarde ancien canal en enregistrement */
    if (ioctl(mixerfd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1)
    {
      if (debug) fprintf (stderr,"Cannot recsrc\n");
      mixer_disable=1;
      close_mixer();
      return -1;
    }

    close_mixer();

    /* Set the list of valid rec sources */
    k=1;
    rec_sound_names[0]="none";
    for (i=0;i<SOUND_MIXER_NRDEVICES; i++)
    {
     if ((1 << i) & recmask)
     {
      rec_sound_names[k]=sound_names[i];
      k++;
     }
    }

    for (;k<(SOUND_MIXER_NRDEVICES+1); k++) rec_sound_names[k]=NULL;

    return 0;
}

int end_mixer()
{
    if (mixer_disable) return -1;
    if (open_mixer()==-1) return -1;

/* met l'ancien input en enregistrement */
    if (ioctl(mixerfd, SOUND_MIXER_WRITE_RECSRC, &recsrc) == -1)
    {
      fprintf(stderr,"Error SOUND_MIXER_WRITE_RECSRC\n");
      mixer_disable=1;
      close_mixer();
      return -1;
    }

    close_mixer();
    return 0;
}

int mute_channel_rec()
{

  int zero=0;

  /* Mute le canal               */
  if (mixer_disable) return -1;
  if (open_mixer()==-1) return -1;

  /* sauvergarde niveau du canal */
  if (ioctl(mixerfd, MIXER_READ(dev_input),&dev_level_input)== -1)
      {
        if (debug) fprintf (stderr,"Error reading channel %s \n",dev_input_name);
        close_mixer();
        return -1;
      }

  if (debug)
   fprintf (stderr,"Get volume channel %d %d\n",dev_level_input >> 8 ,dev_level_input & 0xff );

  /* Mute le canal               */
  if (ioctl(mixerfd, MIXER_WRITE(dev_input), &zero) == -1)
  {
    if (debug) fprintf (stderr,"Error muting channel %s \n",dev_input_name);
    close_mixer();
    return -1;
  }

  /* sauvergarde niveau du canal */
  if (dev_rec_input_gain !=-1)
  {
      if (ioctl(mixerfd, MIXER_READ(dev_rec_input_gain),&rec_level_input)== -1)
      {
        if (debug) fprintf (stderr,"Error reading recording level %s \n",sound_names [dev_rec_input_gain]);
        close_mixer();
        return -1;
      }

      if (debug)
      fprintf (stderr,"Get recording level %d %d\n",rec_level_input >> 8 ,rec_level_input & 0xff );

      if (recording_level==-1)
      {
        switch   (dev_rec_input_gain)
         {
             case SOUND_MIXER_IMIX   :  zero=0x2020; break;
             case SOUND_MIXER_RECLEV :  zero=0x2020; break;
             case SOUND_MIXER_IGAIN  :  zero=0x0101; break;  /* gain 1 ? */
         }
      }
      else
      {
       zero = recording_level |  (recording_level <<8);
      }


      if (ioctl(mixerfd, MIXER_WRITE(dev_rec_input_gain), &zero) == -1)
      {
       if (debug) fprintf (stderr,"Error setting recording channel %s \n",sound_names [dev_rec_input_gain]);
       close_mixer();
       return -1;
      }

      if (debug)
      fprintf (stderr,"set recording level %d %d\n",zero >> 8 ,zero & 0xff );
   }
   /* sauvegarde le niveau de pcm */
   if (ioctl(mixerfd, MIXER_READ(dev_pcm),&old_pcm_level)== -1)
      {
        if (debug) fprintf (stderr,"Error reading channel PCM \n");
        close_mixer();
        return -1;
      }

   /* ecrit le nouveau niveau */
    if (ioctl(mixerfd, MIXER_WRITE(dev_pcm),&dev_level_input)== -1)
      {
        if (debug) fprintf (stderr,"Error writing channel PCM \n");
        close_mixer();
        return -1;
      }

  level_input=dev_level_input & 0x7f;

  record_sound=1;

  close_mixer();
  return 0;
}

int unmute_channel_rec()
{
 if (mixer_disable) return -1;
 if (open_mixer()==-1) return -1;

 if (ioctl(mixerfd, MIXER_READ(dev_pcm),&dev_level_input)== -1)
      {
        if (debug) fprintf (stderr,"Error reading pcm level %s \n",sound_names [dev_rec_input_gain]);
        close_mixer();
        return -1;
      }

 /* ecrit le niveau du canal */
 if (ioctl(mixerfd, MIXER_WRITE(dev_input), &dev_level_input) == -1)
 {
    if (debug) fprintf (stderr,"Error unmuting channel %s \n",dev_input_name);
    close_mixer();
    return -1;
 }

 level_input=dev_level_input & 0x7f;

 if (debug)
   fprintf (stderr,"Restore volume channel %d %d\n",dev_level_input >> 8 ,dev_level_input & 0xff );

 /* Restore le niveau pcm */
    if (ioctl(mixerfd, MIXER_WRITE(dev_pcm),&old_pcm_level)== -1)
      {
        if (debug) fprintf (stderr,"Error writing channel PCM \n");
        close_mixer();
        return -1;
      }

/* restore le niveau d'enregistrement*/
 if (dev_rec_input_gain !=-1)
 {
    if (ioctl(mixerfd, MIXER_WRITE(dev_rec_input_gain), &rec_level_input) == -1)
    {
      if (debug) fprintf (stderr,"Error unmuting channel %s \n",sound_names [dev_rec_input_gain]);
      close_mixer();
      return -1;
    }
 }

 if (debug)
   fprintf (stderr,"Restore record level %d %d\n",rec_level_input >> 8 ,rec_level_input & 0xff );

 record_sound=0;

 close_mixer();
 return 0;

}

int get_mixer_volume (int dev)
{
  int dev_level;

  if (mixer_disable) return -1;
  if (open_mixer()==-1) return -1;

  if (ioctl(mixerfd, MIXER_READ(dev),&dev_level)== -1)
      {
        if (debug) fprintf (stderr,"Error reading channel %s \n",dev_input_name);
        close_mixer();
        return -1;
      }

  if (debug)
   fprintf (stderr,"Get volume channel %d %d\n",dev_level >> 8 ,dev_level & 0xff );

  close_mixer();
  return (dev_level & 0x7f);
}

int set_mixer_volume (int dev, int val)
{
  int dev_level;

  if (mixer_disable) return -1;
  if (open_mixer()==-1) return -1;

  val &= 0x7f;
  dev_level = val | (val << 8);;
  if (-1 == ioctl (mixerfd, MIXER_WRITE (dev), &dev_level))
    {
      perror ("mixer write volume");
      close_mixer();
      return -1;
    }
  close_mixer();
  level_input = val;
  return (dev_level &0x7f);
}

int inc_mixer_volume(void)
{
 int vol;
 int dev;
 if (mixer_disable) return -1;
 if (record_sound) dev=dev_pcm; else dev=dev_input;
 vol=get_mixer_volume (dev);
 vol+=5; if (vol>99) vol=99;
 set_mixer_volume (dev, vol);
 return vol;
}

int dec_mixer_volume(void)
{
 int vol;
 int dev;
 if (mixer_disable) return -1;
 if (record_sound) dev=dev_pcm; else dev=dev_input;
 vol=get_mixer_volume (dev);
 vol-=5; if (vol<0) vol=0;
 set_mixer_volume (dev, vol);
 return vol;
}

int set_mixer_input ( char *name_input)
{
    int i,dev,arg;
    int mute_channel_after=0;

    if (strcmp(dev_input_name,"none")==0) mixer_disable=0;

    if (mixer_disable) return -1;

    if (record_sound)
    {
      unmute_channel_rec();
      mute_channel_after=1;
    }

    if (name_input!=NULL && *name_input!=0)
    {
                              strcpy(dev_input_name,name_input);
    }

    if (strcmp(dev_input_name,"none")==0)
    {
      if (debug) fprintf (stderr,"Mixer feature disabled \n");
      mixer_disable=1;
      return 0;
    }

   if (debug) fprintf (stderr,"Scan mixer record sources\n");
   for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
      if ((1 << i) & recsrc)
      {
       if (debug) fprintf (stderr,"%s  ",sound_names[i]);
      }
   if (debug) fprintf (stderr,"\n");

   for (dev = 0; dev < SOUND_MIXER_NRDEVICES && strcmp(sound_names[dev],dev_input_name); dev++);

   if (dev >= SOUND_MIXER_NRDEVICES)
   {
     if (debug) fprintf (stderr,"Invalid device name %s \n",dev_input_name);
     return -1;
   }

   if (!((1 << dev) & recmask)) {
             fprintf(stderr, "Invalid recording source %s \n",dev_input_name);
             return -1;
   }

  dev_input = dev;

  if (debug) fprintf (stderr, "ok set record source to %s \n",sound_names[dev_input] );

/* met le nouveau input en enregistrement + regle le niveau d'entre  */
// arg = recsrc | (1 << dev_input);
/* Met une seule entre en enregistrement c'est plus correct et plus sain ? */
 if (open_mixer()==-1)
 {
  return -1;
 }
 arg = 1 << dev_input;
 if (ioctl(mixerfd, SOUND_MIXER_WRITE_RECSRC, &arg) == -1)
 {
  fprintf(stderr,"Error SOUND_MIXER_WRITE_RECSRC\n");
  close_mixer();
  return -1;
 }

 if (ioctl(mixerfd, MIXER_READ(dev_input),&level_input)== -1)
      {
        if (debug) fprintf (stderr,"Error reading channel %s \n",dev_input_name);
        close_mixer();
        return -1;
      }

 level_input &= 0x7f;

 close_mixer();

 if (mute_channel_after)
    {
      mute_channel_rec();
    }

 return 0;
}

int set_mixer_input_name ( char *name_input)
{
  if (name_input==NULL || *name_input==0) return -1;
    strcpy(dev_input_name,name_input);
    return 0;
}

char *get_mixer_input_name(void)
{
 return dev_input_name;
}

int get_mixer_input_level(int *level)
{
 if (!mixer_disable) *level=level_input;
 return 0;
}

int set_recording_level(int level)
{
  recording_level = level & 0x7f;
  return 0;
}

int get_recording_level(int *level)
{
  if (level) *level = recording_level;
  return 0;
}

char *get_mixer_device(void)
{
 return mixer_name;
}

int set_mixer_device(char *mixer_device)
{
 if (mixer_device) strcpy(mixer_name,mixer_device);
 return 0;
}




