/*************************************************************************
* nagra.c                                                                *
* Original sources from xawdecode  Thanks to                             *
* Gaston, kamilh, M.Kuhn, Jens, Steffen and Keuleu                       *
* Use by  Maupatz for Taztv  2001                                        *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include "attributes.h"
#include "mmx.h"
#include "taztv.h"
#include "nagra.h"
#include "video.h"
#include "i18n.h"

int table_keys[256];

/* pal support */
int burst_phase;
short colortable[1380 * 4];

unsigned short key_lines[NB_KEYS*NB_COUPLES_MAX];	

int couple_index_nb;					
int disable_decoding=0;

unsigned int couple_index[NB_COUPLES_MAX*200];		
unsigned int correl_lines[NB_COUPLES_MAX*200];	
unsigned short pts_buffer[YSIZE][CX_COUNT];

/* init variables */
int nb_couples;
int *couple_next;
int *couple_first;
int *couple_score;
int *couple_nb;
int number_couples=28;	

void gen_perm(short *perm, int key);

/**********************************Prep tabs*********************************/

int file_exist(char *name)
{
  struct stat buf;

  if (stat(name, &buf) == 0)
    return 1;
  else
    if (errno == ENOENT)
      return 0;
    else
      return 1;
}

void add_couple(int c)
{
  int p,key,q,y1,y2,nb;

  y1=c / YSIZE;
  y2=c % YSIZE;
  couple_index[couple_index_nb]=(y1 << 9) | y2;

  p=couple_first[c];
  while (p!=-1) {
    key=p / nb_couples;
    nb=couple_nb[key]++;
    q=couple_next[p];
    if (nb < number_couples) {
      key_lines[key*number_couples+nb]=couple_index_nb*4;
    }
    p=q;
  }
  couple_index_nb++;
}

int score_couple(int c)
{
  int p,key,score;

  score=0;
  p=couple_first[c];
  while (p!=-1) {
    key=p / nb_couples;
    if (couple_nb[key] < number_couples) score++;
    p=couple_next[p];
  }
  return score;
}

void prep_tab_init_decode(void)
{
  int key,i,j,last_y,y,p,q,score;
  unsigned short perm[YSIZE];
  int nb_tot,score_max;
  int line_min=N_LINE_MIN,line_max=N_LINE_MAX;

  fprintf(stderr,_("Initialising nagra decoding tables - be patient "));
  nb_couples=line_max-line_min;

  couple_next=malloc(sizeof(int)*nb_couples*NB_KEYS);
  couple_first=malloc(YSIZE*YSIZE*sizeof(int));
  couple_score=malloc(YSIZE*YSIZE*sizeof(int));
  couple_nb=malloc(NB_KEYS*sizeof(int));

  for(i=0;i<YSIZE*YSIZE;i++) couple_first[i]=-1;
  for(i=0;i<YSIZE*YSIZE;i++) couple_score[i]=0;

  for(key=0;key<NB_KEYS;key++) {
  gen_perm(perm,key);

    last_y=perm[line_min];
    for(i=line_min+1;i<=line_max;i++) {
      y=perm[i];
      if (y < last_y) j=y*YSIZE+last_y; else j=last_y*YSIZE+y;
      last_y=y;

      q=(key*nb_couples)+i-(line_min+1);
      p=couple_first[j];
      couple_next[q]=p;
      couple_first[j]=q;
      couple_score[j]++;
    }

    if ((key & 1023) == 0) {
      fprintf(stderr,".");
    }
  }

  fprintf(stderr,"\n");

  for(i=0;i<NB_KEYS;i++) couple_nb[i]=0;

  nb_tot=0;
  couple_index_nb=0;
  do {
    score_max=0;
    j=0;
    for(i=0;i<YSIZE*YSIZE;i++) {
      if (couple_score[i] > score_max) {
	score_max=couple_score[i];
	j=i;
      }
    }
    score=score_couple(j);
    if (score > 0) {
      nb_tot+=score;
      add_couple(j);
    }
    couple_score[j]=0;

    if ((nb_tot & 63) == 0) {
      fprintf(stderr,"-");
    }

  } while (nb_tot < (NB_KEYS*number_couples)) ;

  free(couple_nb);
  free(couple_score);
  free(couple_first);
  free(couple_next);

  fprintf(stderr,_("\n[correlation on %d couples]\n"),couple_index_nb);
}


void prep_tabs(void)
{
  FILE* file;
  char name1[128];
  char name2[128];
  
  for (number_couples=NB_COUPLES_MIN;number_couples<=NB_COUPLES_MAX;number_couples+=NB_COUPLES_STEP)
    {
      if ( number_couples> 32)
	number_couples+=NB_COUPLES_STEP;

      sprintf (name1, "%s/couples%i", user_dir,number_couples );
      sprintf (name2, "%s/keyline%i", user_dir,number_couples );

      if (!file_exist(name1) || !file_exist(name2))
	{
	  fprintf (stderr,_("calculating tables for %i couples\n"),number_couples);
	  prep_tab_init_decode();
	  
	  if (!(file=fopen(name1,"w")))
	    {
	      fprintf(stderr, _("Cant't open File %s for writing\n"),name1);
	      exit(-1);
	    }
	  fwrite (couple_index, sizeof(int), couple_index_nb, file);
	  fclose(file);

	  if (!(file=fopen(name2,"w")))
	    {
	      fprintf(stderr,_("Cant't open File %s for writing\n"),name2);
	      exit(-1);
	    }
	  fwrite (key_lines, sizeof(short), number_couples*NB_KEYS, file);
	  fclose(file);
	}
      
    }
 } 

void read_keys(void)
{
  FILE *key_fp;
  int i, tmp;
  char key_file[255];
  
  sprintf(key_file, "%s/%s", user_dir, "key.txt");
  
  if ((key_fp = fopen(key_file, "r")) == (FILE *)NULL)
    {
     fprintf(stderr,_("You will need the file key.txt in the directory %s\n"),user_dir);
     fprintf(stderr,_("Decoder inactived\n"));
     disable_decoding=1;
     return;
    }
  else
    {
      for(i=0;i<256;i++)
	{
	  fscanf(key_fp, "%d\n", &tmp);
          table_keys[i] = tmp;
	}
      fclose(key_fp);
      prep_tabs();
    }
}

/* compute the permutation generated by the 15 bit key 'key' */
void gen_perm(short *perm,int key)
{
  int key_start,key_inc,key_index,i,b;
  int buffer[32];

  key_start=key >> 7;
  key_inc=((key & 0x7f) << 1)  + 1;

  for(i=0;i<32;i++)
    buffer[i]=i;

  key_index=key_start;
  for(i=0;i<255;i++)
    {
      b=table_keys[key_index];
      perm[i]=buffer[b];
      buffer[b]=i+32;

      key_index=(key_index+key_inc) & 0xff;
    }

  for(i=0;i<32;i++)
    {
      perm[255+i]=buffer[i];
    }
}

int init_decode(void)
{
  FILE* file;
  char name[128];


  if (debug) fprintf(stderr,"Reading tables for %i couples\n",number_couples);
                      
  sprintf (name, "%s/couples%i", user_dir, number_couples);
  if (!(file=fopen(name,"r")))
    {
      if (debug) fprintf(stderr,"File %s not found\n",name);
      exit (-1);
    }
  couple_index_nb = fread (couple_index, sizeof(int), sizeof(couple_index)/sizeof(int), file);
  fclose(file);

  sprintf (name, "%s/keyline%i", user_dir, number_couples);
  if (!(file=fopen(name,"r")))
    {
      if (debug) fprintf(stderr,"File %s not found\n",name);
      exit (-1);
    }
  fread (key_lines, sizeof(short), number_couples*NB_KEYS, file);
  fclose(file);
  return 0;  
}

/********************************** end  Prep stuff ****************************/

/********************************** Decoder ***********************************/

/*
 * C version of try_perm 
 */

inline unsigned short *try_perm(void)
{
  int corr_min;
  unsigned short *key_end,*key_min;
  unsigned short *key_ptr;
  int corr;


  corr_min=0x7fffffff;
  key_min=NULL;
  key_ptr=(unsigned short *)key_lines;
  key_end=key_ptr+NB_KEYS*number_couples;

  /* fast correlation on a few lines */
  do
    {
      corr=0;
      { 
	int j;
	for(j=0;j<number_couples;j+=4)
	  {
	    corr+=*(unsigned int *)((unsigned char *)correl_lines + key_ptr[j]);
	    corr+=*(unsigned int *)((unsigned char *)correl_lines + key_ptr[j+1]);
	    corr+=*(unsigned int *)((unsigned char *)correl_lines + key_ptr[j+2]);
	    corr+=*(unsigned int *)((unsigned char *)correl_lines + key_ptr[j+3]);
	    if (corr>corr_min) break;
	  }
      }

      /* test correlation */
      if (corr < corr_min)
	{
	  corr_min=corr;
	  key_min=key_ptr;
	}
      key_ptr+=number_couples;
    }
  while (key_ptr < key_end);

  return key_min;
}

/**********************************************************************************/

int decode_nagra(short *perm, char **lines, int width)
{
  int i,j;
  unsigned short *p;
  int key;
  unsigned short *key_ptr;
  int cx_inc,cx_start;
  static  long long int masque6=0x0000ffff0000ffff;
  int v,y1,y2;
  unsigned short *p1,*p2;

    {                                // phase das pal-burst
    int phase = *(((unsigned char **) lines)[2] + 3);

    burst_phase = 1;
    if (phase < 128)
      burst_phase = 0;

    }

  cx_inc=width/CX_COUNT;
  cx_start=(cx_inc/2)&0x7e;
  p=&pts_buffer[0][0];

  /* preload of the pixels used for the correlation */
  /* YUY2 format */
  cx_inc += cx_inc;
  for(i=0;i<YSIZE;i++)
    {
      unsigned char *q;

      q=((unsigned char **)lines)[i] + cx_start;
      for(j=0;j<CX_COUNT;j+=4)
	{
	  p[0]=q[0];
	  q+=cx_inc;
	  p[1]=q[0];
	  q+=cx_inc;
	  p[2]=q[0];
	  q+=cx_inc;
	  p[3]=q[0];
	  q+=cx_inc;
	  p+=4;
	}
    }

 if (have_mmxext) {

  for (i=0;i<couple_index_nb;i++){

      v  = couple_index[i];
      y1 = v >> 9;
      y2 = v & 0x1ff;
      p1 = &pts_buffer[y1][0];
      p2 = &pts_buffer[y2][0];

      pxor_r2r(mm3,mm3);

      for (j=0;j<CX_COUNT;j+=8) {
                                  movq_m2r(p1[0],mm0);

                               /* Difference valeur absolue */
                                  psadbw_m2r(p2[0],mm0);

                                 /* addition dword */
                                  paddd_r2r(mm0,mm3);

                                  // p1+=4;
                                  // p2+=4;

                                  movq_m2r(p1[4],mm0);

                               /* Difference valeur absolue */
                                  psadbw_m2r(p2[4],mm0);

                                 /* addition dword */
                                  paddd_r2r(mm0,mm3);

                                  p1+=8;
                                  p2+=8;
      }
          movd_r2m(mm3,correl_lines[i]);
  }

  } else {

  /* Precompute the correlation between the lines */
  movq_m2r(masque6,mm6);

  for (i=0;i<couple_index_nb;i++){

      v  = couple_index[i];
      y1 = v >> 9;
      y2 = v & 0x1ff;
      p1 = &pts_buffer[y1][0];
      p2 = &pts_buffer[y2][0];

      pxor_r2r(mm3,mm3);

      for (j=0;j<CX_COUNT;j+=8) {
                                  movq_m2r(p1[0],mm0);
                                  movq_m2r(p2[0],mm1);

                               /* Difference valeur absolue */
                                  movq_r2r(mm0,mm2);
                                  psubusw_r2r(mm1,mm0);
                                  psubusw_r2r(mm2,mm1);
                                  por_r2r(mm1,mm0);

                                 /* addition word */
                                  paddw_r2r(mm0,mm3);

                                //  p1+=4;
                                //  p2+=4;

                                  movq_m2r(p1[4],mm0);
                                  movq_m2r(p2[4],mm1);

                               /* Difference valeur absolue */
                                  movq_r2r(mm0,mm2);
                                  psubusw_r2r(mm1,mm0);
                                  psubusw_r2r(mm2,mm1);
                                  por_r2r(mm1,mm0);

                                 /* addition word */
                                  paddw_r2r(mm0,mm3);

                                  p1+=8;
                                  p2+=8;
      }

      /* addition finale */
      movq_r2r(mm3,mm0);
      psrlq_i2r(32,mm0);
      paddw_r2r(mm0,mm3);
      movq_r2r(mm3,mm0);
      psrlq_i2r(16,mm0);
      paddw_r2r(mm0,mm3);
      pand_r2r(mm6,mm3);
      movd_r2m(mm3,correl_lines[i]);
   }

 }

  emms();

  /* Find the permutation */

  key_ptr=try_perm();
  key=(key_ptr - (unsigned short *)key_lines) / number_couples;
  gen_perm(perm,key);
  return key;
}


int INC_couples(void)
{
        if (disable_decoding) return 0;
	number_couples += NB_COUPLES_STEP;
	if (number_couples > 32) number_couples += NB_COUPLES_STEP; 
	if (number_couples > NB_COUPLES_MAX) number_couples  = NB_COUPLES_MAX;

	init_decode();

	
      return number_couples;
}

int DEC_couples(void)
{
        if (disable_decoding) return 0;
	number_couples -= NB_COUPLES_STEP;
	if (number_couples > 32) number_couples -= NB_COUPLES_STEP; 
	if (number_couples < NB_COUPLES_MIN) number_couples  = NB_COUPLES_MIN;

	init_decode();
	
      return number_couples;

}

int set_couples(int n){
    if (disable_decoding) return 0;
    number_couples=n;
    init_decode();
    return n;
}

int get_couples (int *n){
 *n = number_couples;
 return *n;
}

static char nc_chr [8];

char  *char_get_couples(void) {
    sprintf (nc_chr,"%d",number_couples);
    return nc_chr;
}

/************************* end decoder stuff **********************************/

 int init_nagra(void)
{
  int rc;
  int i;
  double f;
  double f1 = -1.56071095712045105;
  double f2 = 0.1047197551196597746;
  double f3 = 0.7058823529411764706;

  short *p;

   rc=mkdir(user_dir,S_IRWXU);
   if (rc==0) {
     fprintf(stderr,_("You will need the key.txt in the directory  %s\n"),user_dir);
     fprintf(stderr,_("Decoder inactived\n"));
     disable_decoding=0;
     return 0 ;
   }
  read_keys();

  set_couples(NB_COUPLES);

  p = colortable;

  for (i = 0; i < 1380; i++)
    {
      f = (double) ((i >> 2) - 344) * f1 - f2;
      *p = cos (f) * 8192.0;
      *(p + 2) = sin (f) * 8192.0 * f3;
      *(p + 1) = -sin (f) * 8192.0;
      *(p + 3) = cos (f) * 8192.0 * f3;
      if (i & 2)
        {
          *(p + 2) = -*(p + 2);
          *(p + 3) = -*(p + 3);
        }
      if (i & 1)
        {
          *(p + 1) = -*(p + 1);
          *(p + 3) = -*(p + 3);
        }
      p += 4;
    }

 return 0;
 }

 int end_nagra(void){

     return 0;
 }


