/*************************************************************************
* Taztv.c                                                                *
* Maupatz, keuleu     2001                                               *
* taztv is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by   *
* the Free Software Foundation; either version 2 of the License, or      *
* (at your option) any later version.                                    *
*                                                                        *
* Taztv is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
* GNU General Public License for more details.                           *
*                                                                        *
* You should have received a copy of the GNU General Public License      *
* along with this program; if not, write to the Free Software            *
*                                                                        *
*************************************************************************/

#include  "taztv.h"
#include  "attributes.h"
#include  "mmx.h"
#include  "i18n.h"
#include  "nagra.h"
#include  "yuyv2rgb.h"
#include  "video.h"
#include  "x11.h"
#include  "readconfig.h"
#include  "nagraudio.h"
#include  "alevt.h"
#include  "mixer.h"

int have_sse=0;
int have_sse2=0;
int have_mmx=0;
int have_mmxext=0;
int have_3dnow=0;
int have_3dnowext=0;
int cpu_mm_support=0;

int debug=0;
int disable_xshm=0;
int disable_xv=0;
int disable_vm=0;
char user_dir[255];

volatile sig_atomic_t not_end_taztv=1;

void catch_sig_term(int sig){

     not_end_taztv = 0;
     signal(sig,catch_sig_term);

}

void help(){
     printf("%s\n",PACKAGE_STRING);
     printf(_("Valid Options are : \n"));
     printf(_(" -v        : Be verbose\n"));
     printf(_(" -noxv     : Don't use Xv\n"));
     printf(_(" -noxshm   : Don't use XShm\n"));
     printf(_(" -novm     : Don't use video mode\n"));
     printf(_(" -h --help : Print this help\n"));
     printf  (" -video device \n");
     printf  (" -vbi device   \n");
     printf  (" -xvport port  \n");
}

int main(int argc, char **argv){

 int i;

/* user_dir */
strcpy(user_dir,getenv("HOME"));
strcat(user_dir,"/.taztv");

/* locale */

#ifdef ENABLE_NLS
setlocale (LC_ALL, "");
bindtextdomain (PACKAGE, LOCALEDIR);
textdomain (PACKAGE);
#endif

/* Process options */
for (i=1;i<argc;i++) {
     if (strcmp(argv[i],"-noxv")==0){   disable_xv=1;  continue; }
     if (strcmp(argv[i],"-noxshm")==0){ disable_xshm=1;  continue; }
     if (strcmp(argv[i],"-novm")==0){ disable_vm=1;  continue; }
     if (strcmp(argv[i],"-v")==0){      debug=1;   continue; }
     if (strcmp(argv[i],"-h")==0)    { help(); exit(0); }
     if (strcmp(argv[i],"-V")==0)    { help(); exit(0); }
     if (strcmp(argv[i],"--version")==0) { help(); exit(0); }
     if (strcmp(argv[i],"--help")==0){ help(); exit(0); }
     if (strcmp(argv[i],"-vbi")==0){
         i++; if (i>=argc) break;
         strcpy(vbi_name,argv[i]);
         continue;
     }
     if (strcmp(argv[i],"-video")==0){
        i++; if (i>=argc) break;
        strcpy(video_device,argv[i]);
        continue;
     }
     if (strcmp(argv[i],"-xvport")==0){
        i++; if (i>=argc) break;
        user_xvport=atoi(argv[i]);
        continue;
     }

     fprintf(stderr,_("Unknown option : %s\n"), argv[i]);
     help(); exit(0);

}

if (debug) fprintf(stderr,PACKAGE_STRING"\n");

/* test du processeur pour mmx,mmxext, sse, 3dnow */
 cpu_mm_support = mm_support();

 switch (cpu_mm_support) {
                   case 0:
                     fprintf(stderr,_("Need MMX!\n"));
                     exit (-1);
                    break;
                    case 1:
                     if (debug) printf("MMX detect \n");
                    break;
                    case 3:
                     have_mmx = 1;
                     have_mmxext = 1;
                     if (debug) printf("MMX + MMX extended detect \n");
                    break;
                    case 5:
                     have_mmx = 1;
                     have_3dnow=1;
                     if (debug) printf("3DNOW + MMX detect\n");
                    break;
                    case 7:
                     have_mmx = 1;
                     have_3dnow=1;
                     have_mmxext = 1;
                     if (debug) printf("3DNOW + MMX + MMX extended detect\n");
                    break;
                    case 11:
                     have_mmx = 1;
                     have_sse = 1;
                     have_mmxext = 1;
                     if (debug) printf("MMX + MMX extended + SSE detect\n");
                    break;
                    case 23:
                     have_mmx = 1;
                     have_mmxext = 1;
                     have_3dnow=1;
                     have_3dnowext=1;
                     if (debug) printf("3DNOW +3DNOWext + MMX + MMX extended detect\n");
                    break;
                    case 43:
                     have_mmx = 1;
                     have_sse = 1;
                     have_sse2 = 1;
                     have_mmxext = 1;
                     if (debug) printf("MMX + MMX extended + SSE + SSE2 detect\n");
                    break;
                    default:
                     fprintf(stderr,_("What is this processor ?!\n"));
                     exit (-1);
                    break;
 }

/* init */
if (init_nagra()!=0) exit(-1);
if (init_video()!=0){ stop_nagraudio(); end_nagra(); exit(-1); }
if (init_x11()!=0)  { stop_nagraudio(); end_nagra(); end_video(); exit(-1); }
parse_config();

set_last_tv_channel();
init_mixer(NULL);
set_mixer_input(NULL);

/* signal handler */
signal(SIGHUP,catch_sig_term);
signal(SIGINT,catch_sig_term);
signal(SIGQUIT,catch_sig_term);
signal(SIGTERM,catch_sig_term);

/* Process images */
while (not_end_taztv)
 {
   processevent();
   if (!get_img_rb())
   print_rb();
 }

/* end */
stop_nagraudio();
end_nagra();
end_video();
end_x11();
end_mixer();
return 0;
}


